#include <formatio.h>
#include <visa.h>
#include <stdlib.h>  
#include <stdio.h>  
#include <math.h>
#include "avulis.h"							 

#define AVULIS_REVISION  "Rev 3.0, 04/99, CVI 5.0.1" /*Instrument driver revision*/
#define BUFFER_SIZE    512L  /* File I/O buffer size */

/*==== ADVANTEST ULIS Series Spectrum Analyzers: U3641, U3661 and U4941 =====*/ 			        
/*                                                                           */
/* LabWindows/CVI 5.0 Instrument Driver                                      */
/* Original Release: February 1998                                           */
/* By: Denis BIRINGER, Advantest Europe R&D S.A.R.L. Les ULIS France         */
/*     Fax + 33 1 69 18 25 90                                                */
/*     e-mail: d.biringer@erd.advantest.de                                   */
/*                                                                           */
/* Originally written in C                                                   */
/* Modification History:                                                     */
/*                                                                           */
/* April 1998       Added function AVULIS_Set_HighSenseMode, and a goto      */
/*                  Local command in the close function.                     */
/*                  Modified by : Denis BIRINGER, Advantest Europe R&D       */
/*                                                                           */
/* August 1998                                                               */
/*                  Functions modified : Save_File and Search_Scale          */
/*                  New functions : Set_TR_AntennaCorrection,                */
/*                  Set_User_AntennaCorrection                               */
/*                  and Read_AntennaCorrectionStatus                         */
/*                  Modified by : Denis BIRINGER, Advantest Europe R&D 	     */
/*                                                                           */
/* November 1998    Functions modified :                                     */
/*                  - init and defaultInstrSetup to accept U3661             */
/*                  - Set Trace and Read_TraceStatus : Min Hold for U3661    */
/*                  - Set_Span and Set_Freq : avoid that the analyzer sets   */ 
/*                    up the RBW control mode in manual mode                 */
/*                  - Set_Scale : reads the new reference unit               */
/*                  Modified by : Denis BIRINGER, Advantest Europe R&D       */
/*                                                                           */
/* April 1999       Functions modified                                       */
/*                  - init function : sends an empty string before           */
/*                  the first read or write when an RS-232 (Serial)          */
/*                  interface is used.                                       */
/*			        - Set Delta Marker function modified : there was a 	     */
/*                  misfunctioning when the Delta marker was fixed           */
/*                  - Read Multi Marker active : there was a misfunctioning  */
/*                  when the Delta Marker was fixed                          */  
/*                  Modified by: Denis BIRINGER, Advantest Europe R&D.       */
/*===========================================================================*/
 
/*****************************************************************************/
/* INSTRUMENT-DEPENDENT COMMAND ARRAYS                                       */
/*****************************************************************************/
static ViString s_init[] = {"DL0","HD0","TYPE?",""};

static ViString s_freqType[] = {"CF","FA","FB"};         
static ViString s_spanType[] = {"SP","FS","ZS"};         
static ViString s_freqOffset[] = {"FOF","FON"};
static ViString s_freqStepSize[] = {"CS","CA"};
static ViString s_counterResolution[] = {"CN0","CN1","CN2","CN3"};
static ViString s_counterONOFF[] = {"COUNT OFF","COUNT ON"};

static ViString s_refLevel[] = {"RL"};
static ViString s_refUnit[] = {"UB","UM","UU","UE","UW","","KSD","AUNITS W"};
static ViString s_yScale[] = {"DD10DB","DD5DB","DD2DB","DD1DB","LIN","DD?"};
static ViString s_refLevelOffset[] = {"ROF","RON"};
static ViString s_att[] = {"AT","AA"};
static ViString s_highsence[] = {"HS OFF","HS ON"};

static ViString s_RBW[] = {"BA","RB100HZ","RB300HZ","RB1KZ","RB3KZ","RB10KZ",
                           "RB30KZ","RB100KZ","RB300KZ","RB1MZ","RB3MZ"};                    
static ViString s_VBW[] = {"VA","VB10HZ","VB30HZ","VB100HZ","VB300HZ","VB1KZ",
                           "VB3KZ","VB10KZ","VB30KZ","VB100KZ","VB300KZ",
                           "VB1MZ","VB3MZ"};                

static ViString s_All_Coupled[] = {"AL"};

static ViString s_sweepTime[] = {"SW","AS"};
static ViString s_sweepMode[] = {"SI","SN","SR","TS"}; 
static ViString s_windowSweep[] = {"WDOSWP OFF","WDOSWP ON"};
static ViString s_triggerTV[] = {"TVV","TVHNT","TVHPS","TVPOL-","TVPOL+","TVH"};
static ViString s_trigger[] = {"FR","VI","EXT","TRIGSLP-","TRIGSLP+","TRPOSI"};

static ViString s_traceAMode[] = {"AW","AV","AB","AM","","AG"};
static ViString s_averageMode[] = {"AG1","AG0"};  
static ViString s_traceBMode[] = {"BWRITE","BVIEW","BBLANK","BSTORE"};                            
static ViString s_detectorMode[] = {"DTN","DTP","DTG","DTS"};

static ViString s_showDL[] = {"DLF","DLN"};

static ViString s_LLType[] = {"LIMTYP TIME","LIMTYP FREQ"};
static ViString s_deleteLL[] = {"LMTBDEL","LMTADEL"};
static ViString s_setLL[] = {"LMTBIN","LMTAIN"};
static ViString s_showLL1[] = {"LAF","LAN"};
static ViString s_showLL2[] = {"LBF","LBN"}; 
static ViString s_levelX[] = {"LIMPOS ABS","LIMPOS REL"};
static ViString s_levelY[] = {"LIMAPOS ABS","LIMAPOS REL"};
static ViString s_shiftLL[] = {"LIMASFT","LIMSFT"};
static ViString s_LLpassfail[] = {"PFC OFF","PFC ON"};
static ViString s_LLjudgement[] = {"OPF?"};
static ViString s_judgementText[] = {"PASS","UPPER FAIL","LOWER FAIL","UPPER&LOWER FAIL"};

static ViString s_positionWindow[]  = {"WLX","WDX","WUL","WLL"};
static ViString s_showWindow[]  = {"WDO OFF","WDO ON"};

static ViString s_showAC[] = {"CR OFF","CR ON"};
static ViString s_workAC[] = {"CRDEL","CRIN"};
static ViString s_modeAC[] = {"CR LVL","CR ANT"};
static ViString s_antennaAC[] = {"ANT0","ANT1","ANT OFF"};
static ViString s_readAC[] = {"ANT?","CORR?","no antenna correction","dipole antenna correction",
                              "log-periodic antenna correction","user defined antenna correction"}; 

static ViString s_mkrNormal[] = {"MKN"};
static ViString s_mkrDelta[] = {"MKD"};
static ViString s_fixedMkrDelta[] = {"FXF","FXN"};
static ViString s_1_deltaMkr[] = {"REDLT OFF","REDLT ON"};
static ViString s_mkrTrace[] = {"MKTRACE TRB","MKTRACE TRA"};
static ViString s_mkrNDstatus[] = {"markers inactive","only normal marker active",
                                   "normal and delta markers active"};
static ViString s_multiMkrStatus[] = {"MLT"}; 
static ViString s_multiMkrON[] = {"MLN1","MLN2","MLN3","MLN4","MLN5","MLN6"};
static ViString s_multiMkrOFF[] = {"MLF1","MLF2","MLF3","MLF4","MLF5","MLF6"};
static ViString s_deleteMarker[] = {"MO"};
                                  
static ViString s_continuousSearch[] = {"CPF","CPN"};
static ViString s_signalTrack[] = {"SGF","SGN"};
static ViString s_peakRange[] = {"PSN","PSU","PSL","DY"};
static ViString s_peakSearch[] = {"MKPK","NXP","NXL","NXR","MIS","NXM"};
static ViString s_markerDeltaTo[] = {"MTCF","MTSP","MTCS","MTMKS"};
static ViString s_markerStepSize[] = {"MKMKS","MKS?"};
static ViString s_peakTo[] = {"MKRL","MKCF"};

static ViString s_traceMath[] = {"","ACHB","TR2","TR0","TR1"};
static ViString s_NormMode[] = {"AN ON","AR","AI","AN OFF"};
static ViString s_traceAverage[]  = {"AGR","AGS","AGP","AGC"};

static ViString s_readFreq[]= {"CF?","SP?","FA?","FB?","FO?"};
static ViString s_readFreqStepSize[] = {"CS?","CA?"};
static ViString s_readFreqCounter[] = {"COUNT?","MF?"};

static ViString s_readRefLevel[]={"RL?"};
static ViString s_readRefUnit[]={"AUNITS?"};
static ViString s_readUnitName[] = {"dBm","dBmV","dBV","dBVemf","dBpW","","V","W"};
static ViString s_readRefLevelOffset[]={"RO?"};

static ViString s_readSweepTime[] = {"SW?","AS?"};
static ViString s_readSweepMode[] = {"SWM?","Continuous&full","Single&full",
                                     "Continuous&window","Single&window"};
static ViString s_readTrigger[] = {"TM?","Free Run","TV_V","Video","TV_H","External",
                                   "VI?","TVH?","EXT?","TRPOSI?"};

static ViString s_readRBWandVBW[] = {"VB?","RB?","VA?","BA?"};
static ViString s_readAtt[] = {"AT?","AA?"};
static ViString s_readScale[] = {"10dB/","5dB/","2dB/","1dB/","LIN"};
                                  
static ViString s_readMarkerTraceStatus[] = {"MKTRACE?","blank","Trace A","Trace B"};                                  
static ViString s_mkrNDsearch[] = {"MN?","MF?","ML?"};                                    
static ViString s_readMKDoptions[] = {"FX?","REDLT?"};                                   
static ViString s_multiMkrSearch[] = {"MLSF?","MLSL?"};
static ViString s_readmultiMkrStatus[] = {"MLT?"};
static ViString s_multiMkrMessage[] = {"multimarkers inactive","multimarkers active"};                                  
                                  
static ViString s_readPeakOptions[] = {"CP?","SG?","DY?"};
static ViString s_readDisplayLine[] = {"DL?"};
static ViString s_readLLoptions[] = {"LIMAPOS?","LIMPOS?"};
static ViString s_readLLmessage[] = {"Absolute","Relative"};
static ViString s_readMeasWindow[] = {"WLX?","WDX?","WLY?","WDY?","WDO?"};
static ViString s_readDetectorMode[] = {"DM?","Normal","Positive","Negative","Sample"};
static ViString s_readLLstatus[] = {"LMTB?","LMTA?"};
static ViString s_readTraceStatus[] = {"TB?","TA?"};
static ViString s_readTraceMessage[] = {"B_Write","B_View","B_Blank","A_Write","A_View",
                                        "A_Blank","A_norm","A-DL->A","A-B->A","B-A->A",
                                        "A_Max","A_Avg","A_Min"};

static ViString s_textONOFF[] = {"inactive","active"};
static ViString s_readDisplayMode[] = {"absolute","relative"};
                                  
static ViString s_traceBinary[] = {"TBB?","TBA?"};
static ViString s_traceASCII[] = {"TAB?","TAA?"};
static ViString s_traceAccuracy[] = {"TPF","TPC","TP?"};
                                  
static ViString s_calLevel[] = {"CLN","CLF"};                                  
static ViString s_calibration[]  = {"CLA","CLG","IT0","IT1","IT2","IT3","IT4","IT6"};
static ViString s_freqCorrection[] = {"FC OFF","FC ON"};
static ViString s_calCorrection[]  = {"CC OFF","CC ON"};

static ViString s_clearStatusByte[] = {"S2"};
static ViString s_label[] = {"LBDSP OFF","LOF","LBDSP ON","LON/"};

static ViString s_initCard[] = {"MMI /B:/","MMI /A:/"};
static ViString s_copyCard[] = {"ALLCOPY /B: A:/","ALLCOPY /A: B:/"};
static ViString s_selectCard[] = {"CDRB","CDRA"};
static ViString s_recallFile[] = {"RC /B:","RC /A:"};
static ViString s_saveFile[] = {"CSV /B:","CSV /A:","SAVE /B:","SAVE /A:"};
static ViString s_saveSetup[] = {"ITCSSET OFF","ITCSSET ON","ITSET OFF","ITSET ON"};
static ViString s_saveIDlist[] = {"ITCSIDLST OFF","ITCSIDLST ON","ITIDLST OFF","ITIDLST ON"};
static ViString s_saveNormalize[] = {"ITNORM OFF","ITNORM ON"};
static ViString s_saveAntenna[] = {"ITANT OFF","ITANT ON"};
static ViString s_saveTrace[] = {"ITCSTRCOF","ITCSTRCA","ITCSTRCB","ITCSTRCAB",
                                 "ITTRCOF","ITTRCA","ITTRCB","ITTRCAB"};
static ViString s_saveLimit[] = {"ITCSLMTOF","ITCSLMTA","ITCSLMTB","ITCSLMTAB",
                                 "ITLMTOF","ITLMTA","ITLMTB","ITLMTAB"};                                 

static ViString s_reset[]={"IP","MND OFF"};
static ViString s_errorQuery[]={"ERRNO?"};
static ViString s_instrRevision[] = {"REV?"};

static ViString s_freqUnit[] = {"Hz","kHz","MHz","GHz"};
static ViString s_timeUnit[] = {"ns","us","ms","s"};
static ViString s_levelUnit[] = {"DB","DB","DB","DB","DB","","V","W"};
static ViString s_positionUnit[] = {"SC","HZ"};

static ViString s_close[] = {"LOCAL"};

/*****************************************************************************/
/* INSTRUMENT-DEPENDENT STATUS/RANGE STRUCTURE                               */
/*****************************************************************************/

struct AVULIS_stringValPair {
    ViStatus stringVal;
    ViString stringName;
};

struct AVULIS_instrFeatures {
    ViReal64 frequencyLow[3];
    ViReal64 frequencyHigh[3];
    ViReal64 spanLow;
    ViReal64 spanHigh;
    ViReal64 freqOffsetMin;
    ViReal64 freqOffsetMax;
    ViReal64 inputAttMin;
    ViReal64 inputAttMax;
    ViReal64 refLevelMin[8];
    ViReal64 refLevelMax[8];
    ViReal64 refLevelOffsetMin;
    ViReal64 refLevelOffsetMax;
    ViReal64 sweepTimeMini;
    ViReal64 sweepTimeMaxi;
    ViReal64 gatePositionMini;
    ViReal64 gatePositionMaxi;
    ViReal64 gateWidthMini;
    ViReal64 gateWidthMaxi;
    ViInt16  averageCountMini;
    ViInt16  averageCountMaxi;
    ViInt16  DL_LevelMin;
    ViInt16  DL_LevelMax;
    ViInt16  displayMode;
    ViInt16  refUnit;
    ViInt16  span;
    ViInt16  scale;
    ViUInt32 instrumentType;
    ViReal64 deltaYmin;
    ViReal64 deltaYmax;
    ViReal64 calLevelMin;
    ViReal64 calLevelMax;
    ViInt16  MKRMstatus[6]; /*0 if multi-marker inactive, 1 if muti-marker active*/
    ViChar   instrDriverRevision[256];
    
};

typedef struct AVULIS_stringValPair AVULIS_tStringValPair;
    
typedef struct AVULIS_instrFeatures *AVULIS_ptrFeatures;

/*****************************************************************************/
/* UTILITY ROUTINE DECLARATIONS (Non-Exportable Functions)                   */
/*****************************************************************************/

ViBoolean AVULIS_invalidViBooleanRange (ViBoolean val);
ViBoolean AVULIS_invalidViInt16Range (ViInt16 val, ViInt16 min, ViInt16 max);
ViBoolean AVULIS_invalidViInt32Range (ViInt32 val, ViInt32 min, ViInt32 max);
ViBoolean AVULIS_invalidViUInt16Range (ViUInt16 val, ViUInt16 min, ViUInt16 max);
ViBoolean AVULIS_invalidViUInt32Range (ViUInt32 val, ViUInt32 min, ViUInt32 max);
ViBoolean AVULIS_invalidViReal32Range (ViReal32 val, ViReal32 min, ViReal32 max);
ViBoolean AVULIS_invalidViReal64Range (ViReal64 val, ViReal64 min, ViReal64 max);
ViStatus AVULIS_initCleanUp (ViSession openRMSession,ViPSession openInstrSession,
                             ViStatus currentStatus);
ViStatus AVULIS_defaultInstrSetup (ViSession instrSession, ViUInt32 instrType);
ViStatus AVULIS_initReset (ViSession, ViUInt32);
ViStatus AVULIS_Search_Scale (ViSession instrSession, ViPInt16 scale);
                  
/*****************************************************************************/
/* USER-CALLABLE FUNCTIONS (Exportable Functions)                            */
/*****************************************************************************/

/*===========================================================================*/
/* Function: Initialize                                                      */
/* Purpose:  This function opens the instrument, queries the instrument      */
/*           for its ID, and initializes the instrument to a known state.    */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_init (ViRsrc resourceName, ViBoolean IDquery,
                               ViBoolean reset, ViPSession instrSession)
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViSession rmSession = 0;
    ViUInt32 retCnt = 0;
    ViByte Buffer[BUFFER_SIZE];
    ViByte interim[5];
    ViUInt32 AVULIS_instrType,retCount;
    ViByte cable;
    
    /*- Check input parameter ranges ----------------------------------------*/

    if (AVULIS_invalidViBooleanRange (IDquery))
        return VI_ERROR_PARAMETER2;
        
    if (AVULIS_invalidViBooleanRange (reset))
        return VI_ERROR_PARAMETER3;

    /*- Open instrument session ---------------------------------------------*/

    if ((AVULIS_status = viOpenDefaultRM (&rmSession)) < 0)
        return AVULIS_status;

    if ((AVULIS_status = viOpen (rmSession, resourceName, VI_NULL,VI_NULL, instrSession)) < 0) {
        viClose (rmSession);
        return AVULIS_status;
    }

    /*- Configure VISA Formatted I/O ----------------------------------------*/
    cable = resourceName[0]; /* GPIB or RS-232 cable */
    
    if (cable=='G') { /*GPIB cable*/

        if ((AVULIS_status = viSetAttribute (*instrSession, VI_ATTR_TMO_VALUE, 10000)) < 0)
            return AVULIS_initCleanUp (rmSession, instrSession, AVULIS_status);
            
        if ((AVULIS_status = viSetBuf (*instrSession, VI_READ_BUF|VI_WRITE_BUF, 4000)) < 0)
            return AVULIS_initCleanUp (rmSession, instrSession, AVULIS_status);
            
        if ((AVULIS_status = viSetAttribute (*instrSession, VI_ATTR_WR_BUF_OPER_MODE,
                                             VI_FLUSH_ON_ACCESS))   < 0)
            return AVULIS_initCleanUp (rmSession, instrSession, AVULIS_status);
            
        if ((AVULIS_status = viSetAttribute (*instrSession, VI_ATTR_RD_BUF_OPER_MODE,
                                             VI_FLUSH_ON_ACCESS))   < 0)
            return AVULIS_initCleanUp (rmSession, instrSession, AVULIS_status);
        
        Fmt (Buffer,"%s<%s",s_init[0]);     
        if ((AVULIS_status = viWrite(*instrSession,Buffer,NumFmtdBytes(),&retCount)) < 0)
            return AVULIS_initCleanUp (rmSession, instrSession, AVULIS_status);
            
    }
    
    else { /*RS-232 cable*/
        /* Serial Interface initialized to 8 databits, 1 stopbit, no parity, */
        /* baud rate = 19200, flow control = XonXoff and timeout = 20000 ms  */
        
        if ((AVULIS_status = viSetAttribute (*instrSession, VI_ATTR_ASRL_DATA_BITS, 8))<0)
            return AVULIS_initCleanUp (rmSession, instrSession, AVULIS_status);
    
        if ((AVULIS_status = viSetAttribute (*instrSession, VI_ATTR_ASRL_STOP_BITS,VI_ASRL_STOP_ONE))<0)
            return AVULIS_initCleanUp (rmSession, instrSession, AVULIS_status);
            
        if ((AVULIS_status = viSetAttribute (*instrSession, VI_ATTR_ASRL_PARITY, VI_ASRL_PAR_NONE))<0)
            return AVULIS_status;
        
        if ((AVULIS_status = viSetAttribute (*instrSession, VI_ATTR_ASRL_BAUD, 19200))<0)
            return AVULIS_status;
        
        if ((AVULIS_status = viSetAttribute (*instrSession, VI_ATTR_ASRL_FLOW_CNTRL,VI_ASRL_FLOW_XON_XOFF))<0)
            return AVULIS_status;                 
        
        if ((AVULIS_status = viSetAttribute (*instrSession, VI_ATTR_TMO_VALUE, 20000)) < 0)
            return AVULIS_status;                     
   
        if ((AVULIS_status = viSetBuf (*instrSession, VI_ASRL_IN_BUF|VI_ASRL_OUT_BUF, 512)) < 0)
            return AVULIS_initCleanUp (rmSession, instrSession, AVULIS_status);
   
        if ((AVULIS_status = viSetBuf (*instrSession, VI_READ_BUF|VI_WRITE_BUF, 4000)) < 0)
            return AVULIS_initCleanUp (rmSession, instrSession, AVULIS_status);
   
        if ((AVULIS_status = viSetAttribute (*instrSession, VI_ATTR_WR_BUF_OPER_MODE,VI_FLUSH_ON_ACCESS))<0)
            return AVULIS_initCleanUp (rmSession, instrSession, AVULIS_status);
    
        if ((AVULIS_status = viSetAttribute (*instrSession, VI_ATTR_RD_BUF_OPER_MODE,VI_FLUSH_ON_ACCESS)) < 0)
            return AVULIS_initCleanUp (rmSession, instrSession, AVULIS_status);
 
        if ((AVULIS_status = viSetAttribute (*instrSession, VI_ATTR_TERMCHAR, '\n')) < 0)
            return AVULIS_initCleanUp (rmSession, instrSession, AVULIS_status);
 
        if ((AVULIS_status = viSetAttribute (*instrSession, VI_ATTR_ASRL_END_IN,VI_ASRL_END_TERMCHAR)) < 0)
            return AVULIS_initCleanUp (rmSession, instrSession, AVULIS_status);
   
        if ((AVULIS_status = viSetAttribute (*instrSession, VI_ATTR_ASRL_END_OUT,VI_ASRL_END_NONE)) < 0)
            return AVULIS_initCleanUp (rmSession, instrSession, AVULIS_status); 
            
        // sends an empty string to the analyzer to init the RS-232 link //
        Fmt (Buffer,"%s<%s\r\n",s_init[3]);    
        if ((AVULIS_status = viWrite(*instrSession,Buffer,NumFmtdBytes(),&retCount)) < 0)
            return AVULIS_initCleanUp (rmSession, instrSession, AVULIS_status);    
            
    }       

    /*- Identification Query -------------------------------------------------*/
    
    if (IDquery) {  /* identification executed */
    
        Fmt (Buffer,"%s<%s,%s\r\n",s_init[1],s_init[2]);     
        if ((AVULIS_status = viWrite(*instrSession,Buffer,NumFmtdBytes(),&retCount)) < 0)
            return AVULIS_initCleanUp (rmSession, instrSession, AVULIS_status);
    
        if ((AVULIS_status = viRead (*instrSession, Buffer, BUFFER_SIZE, &retCnt)) < 0)
            return AVULIS_initCleanUp (rmSession, instrSession, AVULIS_status);
         
        if (Buffer[0] != 'U')  
            return AVULIS_initCleanUp (rmSession, instrSession, VI_ERROR_FAIL_ID_QUERY);
         
        interim[0] = Buffer[1];
        interim[1] = Buffer[2];
        interim[2] = Buffer[3];
        interim[3] = Buffer[4];
        interim[4] = Buffer[5];
      
        if (Scan(interim, "%s>%d[b4]", &AVULIS_instrType)!=1) {
            AVULIS_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
            return AVULIS_status;
        } 
        
    }
    
    else { /* no identification executed */
        
        Fmt (Buffer,"%s<%s\r\n",s_init[1]);     
        if ((AVULIS_status = viWrite(*instrSession,Buffer,NumFmtdBytes(),&retCount)) < 0)
            return AVULIS_initCleanUp (rmSession, instrSession, AVULIS_status);
        
        AVULIS_instrType = 3641 ;
        
    }   
     
   
    /*- Reset instrument ----------------------------------------------------*/
    if (reset) {
        if ((AVULIS_status = AVULIS_initReset (*instrSession,AVULIS_instrType)) < 0)
            return AVULIS_initCleanUp (rmSession, instrSession, AVULIS_status);
    }      
    else { /*- Send Default Instrument Setup ---------------------------------*/ 
        if ((AVULIS_status = AVULIS_defaultInstrSetup (*instrSession,AVULIS_instrType)) < 0)
            return AVULIS_initCleanUp (rmSession, instrSession, AVULIS_status);
    }
    
    return AVULIS_status;
}

/*===========================================================================*/
/* Function : configure and read                                             */
/* Purpose : application function                                            */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_configRead(ViSession instrID,ViBoolean trace,ViInt16 peakSearch,
                                    ViPInt16 traceAccuracy,ViInt16 _VI_FAR traceArray[],
                                    ViPReal64 sweepTime,ViChar _VI_FAR sweepUnit[],
                                    ViPReal64 MKRNposition,ViPReal64 MKRNlevel)
{
    ViStatus AVULIS_status = VI_SUCCESS;
    AVULIS_ptrFeatures featuresPtr;
    ViInt16 status,control;
    ViReal64 sweepValue;
    
    if ((AVULIS_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVULIS_status;
       
    if (AVULIS_invalidViBooleanRange (trace))             
        return VI_ERROR_PARAMETER2;
        
    if (AVULIS_invalidViInt16Range (peakSearch,0,5))             
        return VI_ERROR_PARAMETER3;
        
    if ((AVULIS_status = AVULIS_Read_TraceAccuracy (instrID, traceAccuracy))<0)
        return AVULIS_status;
    
    if ((AVULIS_status = AVULIS_Read_TraceData (instrID, trace,1,traceArray))<0)
        return AVULIS_status;
    
    if ((AVULIS_status = AVULIS_Search_Peak (instrID, peakSearch)) <0)
        return AVULIS_status;   
       
    if ((AVULIS_status = AVULIS_Read_SweepTime (instrID, &sweepValue, &control))<0)
        return AVULIS_status;
       
    if ((AVULIS_status = AVULIS_Convert_Time (instrID, sweepValue, sweepTime, sweepUnit))<0)
        return AVULIS_status;
       
    if ((AVULIS_status = AVULIS_Read_NormalMarker (instrID, MKRNposition, MKRNlevel))<0)
        return AVULIS_status;   
       
    return AVULIS_status;
} 

/*===========================================================================*/
/* Function: Set Frequency                                                   */           
/* Purpose:  sets up center, start or stop frequency.                        */           
/*===========================================================================*/           
ViStatus _VI_FUNC AVULIS_Set_Freq (ViSession instrID, ViInt16 freqType,
                                                      ViReal64 freqValue)
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    AVULIS_ptrFeatures featuresPtr;
    ViUInt32 retCount=0;
    ViInt16 RBWcontrolType;
    ViReal64 span, RBWvalue;

    if ((AVULIS_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVULIS_status;
        
    if (AVULIS_invalidViInt16Range (freqType, 0, 2))             
        return VI_ERROR_PARAMETER2;
        
    if (AVULIS_invalidViReal64Range (freqValue, featuresPtr->frequencyLow[freqType],
                                                featuresPtr->frequencyHigh[freqType]))             
        return VI_ERROR_PARAMETER3;
        
    // when the span becomes equal to zero (possible if freq start = freq stop),
    // RBW control type becomes equal to Manual even if it was equal to Auto
    // We have to correct this problem.
    AVULIS_Read_RBWandVBW (instrID, VI_ON, &RBWvalue, &RBWcontrolType);
    
    Fmt (wrtBuffer,"%s<%s%fHZ\r\n",s_freqType[freqType],freqValue);     
    if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return AVULIS_status;
    
    // looks for the span value    
    AVULIS_Read_FreqSettings (instrID, 1, &span);
    if (span==0) {
    	featuresPtr->span=0;
    	if (RBWcontrolType==1) AVULIS_Set_RBW (instrID, 0);
    }	
    else featuresPtr->span=1;
                                  
    return AVULIS_status;
}

/*===========================================================================*/
/* Function: Set Frequency Span                                              */           
/* Purpose:  sets up the frequency span.                                     */           
/*===========================================================================*/           
ViStatus _VI_FUNC AVULIS_Set_Span (ViSession instrID, ViInt16 spanType,
                                                      ViReal64 spanValue)
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    AVULIS_ptrFeatures featuresPtr;
    ViUInt32 retCount=0;
    ViInt16 RBWcontrolType;
    ViReal64 RBWvalue;
    
    if ((AVULIS_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVULIS_status;

    if (AVULIS_invalidViInt16Range (spanType, 0, 2))             
        return VI_ERROR_PARAMETER2;
        
    // when the span becomes equal to zero, RBW control type becomes equal to Manual
    // even if it was equal to Auto. We have to correct this problem
    AVULIS_Read_RBWandVBW (instrID, VI_ON, &RBWvalue, &RBWcontrolType);
       
    if (spanType==0) {  /* Manual Span Setting */ 
        if (AVULIS_invalidViReal64Range (spanValue, featuresPtr->spanLow, 
                                                    featuresPtr->spanHigh))             
            return VI_ERROR_PARAMETER3;
        Fmt (wrtBuffer,"%s<%s%fHZ\r\n",s_spanType[spanType],spanValue);     
        if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return AVULIS_status;
    }
    else { /*full span or zero span*/
        Fmt (wrtBuffer,"%s<%s\r\n",s_spanType[spanType]);     
        if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return AVULIS_status;
    }
    
    if ((spanType==2)||(spanValue==0)) {
    	featuresPtr->span=0;  /*span null*/
    	if (RBWcontrolType==1) AVULIS_Set_RBW (instrID, 0);  
    }	
    else featuresPtr->span=1; /*span not null*/
     
    return AVULIS_status;
}                         

/*===========================================================================*/
/* Function: Set Frequency Offset                                            */           
/* Purpose:  sets up frequency offset and activates frequency offset         */           
/*===========================================================================*/           
ViStatus _VI_FUNC AVULIS_Set_FreqOffset (ViSession instrID, ViBoolean activation,
                                                            ViReal64 freqOffset)
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    AVULIS_ptrFeatures featuresPtr;
    ViUInt32 retCount=0;
    
    if ((AVULIS_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVULIS_status;
       
    if (AVULIS_invalidViBooleanRange (activation))             
        return VI_ERROR_PARAMETER2;   

    if (activation == VI_ON) { /*activation*/
        if (AVULIS_invalidViReal64Range (freqOffset,featuresPtr->freqOffsetMin,
                                                    featuresPtr->freqOffsetMax))             
            return VI_ERROR_PARAMETER3;
        Fmt (wrtBuffer,"%s<%s%fHZ\r\n", s_freqOffset[activation], freqOffset);     
        if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return AVULIS_status;
    }
    else { /*desactivation*/
        Fmt (wrtBuffer,"%s<%s\r\n",s_freqOffset[activation]);     
        if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return AVULIS_status;
    }
     
    return AVULIS_status;
}

/*===========================================================================*/
/* Function: Set Frequency Step Size                                         */           
/* Purpose:  sets up and activates or deactivates Frequency Step Size        */           
/*===========================================================================*/           
ViStatus _VI_FUNC AVULIS_Set_FreqStepSize (ViSession instrID, ViBoolean controlType,
                                                              ViReal64 freqStepSize)
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    AVULIS_ptrFeatures featuresPtr;
    ViUInt32 retCount=0;
    
    if ((AVULIS_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVULIS_status;

    if (AVULIS_invalidViBooleanRange (controlType))             
        return VI_ERROR_PARAMETER2;
       
    if (controlType == VI_ON) { /*step size chosen by the analyzer*/ 
        Fmt (wrtBuffer,"%s<%s\r\n", s_freqStepSize[controlType]);     
        if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return AVULIS_status;
    }
    else { /*step size chosen by the user*/   
        if (AVULIS_invalidViReal64Range (freqStepSize,featuresPtr->spanLow,
                                                      featuresPtr->spanHigh))             
            return VI_ERROR_PARAMETER3;   
        Fmt (wrtBuffer,"%s<%s%fHZ\r\n",s_freqStepSize[controlType],freqStepSize);     
        if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return AVULIS_status;
    }
     
    return AVULIS_status;
}

/*===========================================================================*/
/* Function: Set Frequency Counter                                           */
/* Purpose:  Sets the counter ON / OFF or sets the counter resolution        */
/*===========================================================================*/                               
ViStatus _VI_FUNC AVULIS_Set_FreqCounter (ViSession instrID, ViBoolean activation,
                                                             ViInt16 resolution)
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    AVULIS_ptrFeatures featuresPtr;
    ViUInt32 retCount=0;
    
    if (AVULIS_invalidViBooleanRange (activation))             
        return VI_ERROR_PARAMETER2;
       
    if (AVULIS_invalidViInt16Range (resolution,0,3))             
        return VI_ERROR_PARAMETER3;
       
    if (activation == VI_ON) {
        Fmt (wrtBuffer,"%s<%s\r\n", s_counterResolution[resolution]);     
        if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return AVULIS_status; 
    }

    Fmt (wrtBuffer,"%s<%s\r\n", s_counterONOFF[activation]);     
    if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return AVULIS_status;

    return AVULIS_status;
}

/*===========================================================================*/
/* Function: Set Reference Level                                             */           
/* Purpose:  sets up the reference level (Y axis).                           */           
/*===========================================================================*/           
ViStatus _VI_FUNC AVULIS_Set_RefLevel (ViSession instrID, ViReal64 refLevel)
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    AVULIS_ptrFeatures featuresPtr;
    ViInt16 unit=0;
    ViUInt32 retCount=0;
    
    if ((AVULIS_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVULIS_status;
    
    /* sets up the reference level */
    
    if (AVULIS_invalidViReal64Range (refLevel,featuresPtr->refLevelMin[featuresPtr->refUnit],
                                              featuresPtr->refLevelMax[featuresPtr->refUnit]))             
        return VI_ERROR_PARAMETER2;
        
    Fmt (Buffer,"%s<%s%f%s\r\n",s_refLevel[0],refLevel,s_levelUnit[featuresPtr->refUnit]);     
        if ((AVULIS_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount)) < 0)
            return AVULIS_status;
    
    return AVULIS_status;
}

/*===========================================================================*/
/* Function: Set Reference Unit                                              */           
/* Purpose:  Set up the unit of the reference level.                         */           
/*===========================================================================*/           
ViStatus _VI_FUNC AVULIS_Set_RefUnit (ViSession instrID, ViInt16 refUnit)
{   
    ViStatus AVULIS_status = VI_SUCCESS;  
    ViByte wrtBuffer[BUFFER_SIZE];
    AVULIS_ptrFeatures featuresPtr;
    ViUInt32 retCount=0;
    
    if ((AVULIS_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVULIS_status;
    
    if (AVULIS_invalidViInt16Range (refUnit,0,7))             
        return VI_ERROR_PARAMETER2;
    
    featuresPtr->refUnit = refUnit;    
    
    Fmt (wrtBuffer,"%s<%s\r\n",s_refUnit[refUnit]);     
    if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return AVULIS_status;
       
    return AVULIS_status;
}

/*===========================================================================*/
/* Function: Set Scale                                                       */           
/* Purpose:  Set up the scale of the Y axis .                                */           
/*===========================================================================*/           
ViStatus _VI_FUNC AVULIS_Set_Scale (ViSession instrID, ViInt16 scale)
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    AVULIS_ptrFeatures featuresPtr;
    ViUInt32 retCount=0;
    ViChar unitName[10];
    
    if ((AVULIS_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVULIS_status;

    if (AVULIS_invalidViInt16Range (scale,0,4))             
        return VI_ERROR_PARAMETER2;
        
    Fmt (wrtBuffer,"%s<%s\r\n",s_yScale[scale]);     
    if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return AVULIS_status;
       
    featuresPtr->scale = scale;
    
    // looks for the reference unit. it may have been modified by the scale change
    AVULIS_Read_RefUnit (instrID, &featuresPtr->refUnit,unitName); 
                                  
    return AVULIS_status;
}

/*===========================================================================*/
/* Function: Set Reference Level Offset                                      */           
/* Purpose:  Set up and activates the reference level offset.                */           
/*===========================================================================*/           
ViStatus _VI_FUNC AVULIS_Set_RefOffset (ViSession instrID, ViBoolean activation,
                                                           ViReal64 refLevelOffset)
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    AVULIS_ptrFeatures featuresPtr;
    ViUInt32 retCount=0;
    
    if ((AVULIS_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVULIS_status;
       
    if (AVULIS_invalidViBooleanRange (activation))             
        return VI_ERROR_PARAMETER2;   

    if (activation == VI_ON) {  /* activation */
        if (AVULIS_invalidViReal64Range (refLevelOffset,featuresPtr->refLevelOffsetMin,
                                                        featuresPtr->refLevelOffsetMax))             
            return VI_ERROR_PARAMETER3;
        Fmt (wrtBuffer,"%s<%s%fDB\r\n", s_refLevelOffset[activation], refLevelOffset);     
        if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return AVULIS_status;
    }
    else { /* desactivation */
        Fmt (wrtBuffer,"%s<%s\r\n",s_refLevelOffset[activation]);     
        if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return AVULIS_status;
    }
    
    return AVULIS_status;
}

/*===========================================================================*/
/* Function: Set Input Attenuation                                           */           
/* Purpose:  Set up the Input attenuation.                                   */           
/*===========================================================================*/           
ViStatus _VI_FUNC AVULIS_Set_Att (ViSession instrID, ViBoolean controlType,
                                                     ViReal64 inputAttValue)
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    AVULIS_ptrFeatures featuresPtr;
    ViUInt32 retCount=0;
    
    if ((AVULIS_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVULIS_status;
       
    if (AVULIS_invalidViBooleanRange (controlType))             
        return VI_ERROR_PARAMETER2;   

    if (controlType == VI_ON) { /* attenuation chosen by the analyzer */ 
        Fmt (wrtBuffer,"%s<%s\r\n", s_att[controlType]);     
        if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
             return AVULIS_status;
    }
    else { /* attenuation chosen by the user */
        if (AVULIS_invalidViReal64Range (inputAttValue, featuresPtr->inputAttMin,
                                                        featuresPtr->inputAttMax))             
            return VI_ERROR_PARAMETER3; 
        Fmt (wrtBuffer,"%s<%s%fDB\r\n",s_att[controlType],inputAttValue);     
        if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return AVULIS_status;
    }
     
    return AVULIS_status;
}

/*===========================================================================*/
/* Function: Set High Sence Mode                                             */           
/* Purpose:  Set up the High Sensitive Mode.                                 */           
/*===========================================================================*/           
ViStatus _VI_FUNC AVULIS_Set_HighSenseMode (ViSession instrID, ViBoolean mode)
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (AVULIS_invalidViBooleanRange (mode))             
        return VI_ERROR_PARAMETER2;   

    Fmt (wrtBuffer,"%s<%s\r\n", s_highsence[mode]);     
    if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return AVULIS_status;
    
    return AVULIS_status;
}

/*===========================================================================*/
/* Function: Set RBW                                                         */           
/* Purpose:  Set up the Resolution Bandwidth.                                */
/*===========================================================================*/           
ViStatus _VI_FUNC AVULIS_Set_RBW (ViSession instrID, ViInt16 RBWsetting)
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;

    if (AVULIS_invalidViInt16Range (RBWsetting,0,10))             
        return VI_ERROR_PARAMETER2;

    Fmt (wrtBuffer,"%s<%s\r\n",s_RBW[RBWsetting]);     
    if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return AVULIS_status;
       
    return AVULIS_status;
}

/*===========================================================================*/
/* Function: Set VBW                                                         */           
/* Purpose:  Set up the Video Bandwidth.                                     */
/*===========================================================================*/           
ViStatus _VI_FUNC AVULIS_Set_VBW (ViSession instrID, ViInt16 VBWsetting)
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;

    if (AVULIS_invalidViInt16Range (VBWsetting,0,12))             
        return VI_ERROR_PARAMETER2;
        
    Fmt (wrtBuffer,"%s<%s\r\n",s_VBW[VBWsetting]);     
    if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return AVULIS_status;    

    return AVULIS_status;
}

/*===========================================================================*/
/* Function: Set All Auto Couple                                             */           
/* Purpose:  Sets all couple functions to automatic mode                     */
/*===========================================================================*/           
ViStatus _VI_FUNC AVULIS_Set_All_Auto_Couple (ViSession instrID)
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    AVULIS_ptrFeatures featuresPtr;
    ViUInt32 retCount=0;

    Fmt (wrtBuffer,"%s<%s\r\n",s_All_Coupled[0]);     
    if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return AVULIS_status; 
    
    return AVULIS_status;
}

/*===========================================================================*/
/* Function: Set sweep time                                                  */           
/* Purpose:  Sets the sweep time (sec)                                       */
/*===========================================================================*/           
ViStatus _VI_FUNC AVULIS_Set_SweepTime (ViSession instrID, ViBoolean controlType,
                                                           ViReal64 sweepTime)
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    AVULIS_ptrFeatures featuresPtr;
    ViUInt32 retCount=0;

    if ((AVULIS_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVULIS_status;
       
    if (AVULIS_invalidViBooleanRange (controlType))             
        return VI_ERROR_PARAMETER2;
        
    if (controlType == VI_ON) { /*sweep time is chosen by the analyzer*/  
        Fmt (wrtBuffer,"%s<%s\r\n",s_sweepTime[controlType]);     
        if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return AVULIS_status;  
    }
    else { /*sweep time is chosen by the user*/ 
        if (AVULIS_invalidViReal64Range (sweepTime,featuresPtr->sweepTimeMini,
                                                   featuresPtr->sweepTimeMaxi))             
            return VI_ERROR_PARAMETER3;
        Fmt (wrtBuffer,"%s<%s%fSC\r\n",s_sweepTime[controlType],sweepTime);     
        if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return AVULIS_status;  
    }
    
    return AVULIS_status;
}

/*===========================================================================*/
/* Function: Set sweep mode                                                  */           
/* Purpose:  Sets the sweep mode: continuous / single, window sweep on / off */
/*===========================================================================*/           
ViStatus _VI_FUNC AVULIS_Set_SweepMode (ViSession instrID, ViBoolean sweepMode,
                                                           ViBoolean windowSweep)
{
    ViStatus AVULIS_status = VI_SUCCESS;  
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;

    if (AVULIS_invalidViBooleanRange (sweepMode))             
        return VI_ERROR_PARAMETER2;
       
    if (AVULIS_invalidViBooleanRange (windowSweep))             
        return VI_ERROR_PARAMETER3;
     
    Fmt (wrtBuffer,"%s<%s,%s\r\n",s_sweepMode[sweepMode],s_windowSweep[windowSweep]);     
    if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return AVULIS_status;
     
    return AVULIS_status;
}

/*===========================================================================*/
/* Function: Set trigger                                                     */           
/* Purpose:  Defines the trigger source (except TV trigger, the polarity of  */
/*           the video and external triggers, the levels at which the video  */
/*           and external trigger occurs                                     */
/*===========================================================================*/           
ViStatus _VI_FUNC AVULIS_Set_Trigger(ViSession instrID, ViInt16 source,
                                                        ViBoolean polarity,
                                                        ViInt32 videoLevel,
                                                        ViReal64 externalLevel)
{
    ViStatus AVULIS_status = VI_SUCCESS;  
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;

    if (AVULIS_invalidViInt16Range (source,0,2))             
        return VI_ERROR_PARAMETER2;
        
    if (AVULIS_invalidViBooleanRange (polarity))             
        return VI_ERROR_PARAMETER3;
        
    if (source==0) { /* free run */
        Fmt (wrtBuffer,"%s<%s\r\n",s_trigger[source]);     
        if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return AVULIS_status; 
    }
    
    else if (source==1) { /* video trigger */
        if (AVULIS_invalidViInt16Range (videoLevel,0,100))             
            return VI_ERROR_PARAMETER4;
        Fmt (wrtBuffer,"%s<%s%d[b4]ENT,%s\r\n",s_trigger[source],videoLevel,s_trigger[3+polarity]);     
        if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return AVULIS_status;
    }
    
    else if (source==2) {  /* external trigger */
        if (AVULIS_invalidViInt16Range (externalLevel,0,5))             
            return VI_ERROR_PARAMETER5;
        Fmt (wrtBuffer,"%s<%s%fV,%s\r\n",s_trigger[source],externalLevel,s_trigger[3+polarity]);     
        if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return AVULIS_status;
    }       

    return AVULIS_status;
}
 
/*===========================================================================*/
/* Function: Set trigger position                                            */           
/* Purpose:  Defines the position along the x-axis at which the trigger      */
/*           occurs                                                          */
/*===========================================================================*/           
ViStatus _VI_FUNC AVULIS_Set_TriggerPosition(ViSession instrID, ViInt16 position)
{
    ViStatus AVULIS_status = VI_SUCCESS;  
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;

    if (AVULIS_invalidViInt16Range (position,0,100))             
        return VI_ERROR_PARAMETER2;
        
    Fmt (wrtBuffer,"%s<%s%d[b2]ENT\r\n",s_trigger[5],position);     
    if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return AVULIS_status; 
    
    return AVULIS_status;
} 

/*===========================================================================*/
/* Function: Set trigger TV                                                  */           
/* Purpose:  Defines the trigger TV source,the polarity of this trigger      */
/*           and the line number at which the trigger occurs                 */
/*===========================================================================*/           
ViStatus _VI_FUNC AVULIS_Set_TriggerTV(ViSession instrID, ViInt16 triggerSource,
                                                          ViBoolean triggerPolarity,
                                                          ViInt16 lineNumber)
{
    ViStatus AVULIS_status = VI_SUCCESS;  
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;

    if (AVULIS_invalidViInt16Range (triggerSource,0,2))             
        return VI_ERROR_PARAMETER2;
        
    if (AVULIS_invalidViBooleanRange (triggerPolarity))             
        return VI_ERROR_PARAMETER3;
        
    if (triggerSource != 0) { /* TV-H */
        if (AVULIS_invalidViInt16Range (lineNumber,1,625))             
            return VI_ERROR_PARAMETER4;
        Fmt (wrtBuffer,"%s<%s,%s,%s%d[b2]ENT\r\n",s_triggerTV[triggerSource],
                                                  s_triggerTV[3+triggerPolarity],
                                                  s_triggerTV[5],lineNumber);     
    }
    else { /* TV-V */
        Fmt (wrtBuffer,"%s<%s,%s\r\n",s_triggerTV[triggerSource],
                                      s_triggerTV[3+triggerPolarity]);
    }
                                                  
    if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return AVULIS_status; 

    return AVULIS_status;
}

/*===========================================================================*/
/* Function: Set Trace A Mode                                                */           
/* Purpose:  Defines Trace A mode (Write, View, Max Hold ...)                */
/*===========================================================================*/           
ViStatus _VI_FUNC AVULIS_Set_TraceAMode (ViSession instrID, ViInt16 traceAMode,
                                                            ViInt16 averageCount,
                                                            ViBoolean averageMode)
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    AVULIS_ptrFeatures featuresPtr;
    ViUInt32 retCount=0;

    if ((AVULIS_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVULIS_status;

    if (AVULIS_invalidViInt16Range (traceAMode,0,5))             
        return VI_ERROR_PARAMETER2;
        
    if (featuresPtr->instrumentType==3661) {
    	s_traceAMode[4] = "AMIN";
    }
    else s_traceAMode[4] = "";
    
       
    if (traceAMode != 5) { /* set trace mode except trace average */ 
        Fmt (wrtBuffer,"%s<%s\r\n",s_traceAMode[traceAMode]);   
        if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
            return AVULIS_status;
    }
    else { /* set trace average: some parameters are needed */
        if (AVULIS_invalidViInt16Range (averageCount,featuresPtr->averageCountMini,
                                                     featuresPtr->averageCountMaxi))             
            return VI_ERROR_PARAMETER3;
        if (AVULIS_invalidViBooleanRange (averageMode))             
            return VI_ERROR_PARAMETER4;
        Fmt (wrtBuffer,"%s<%s,%s%d[b2]ENT\r\n",s_averageMode[averageMode],
                                               s_traceAMode[traceAMode],averageCount);   
        if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
            return AVULIS_status;
    }
    
    return AVULIS_status;
}

/*===========================================================================*/
/* Function: Set Trace B Mode                                                */           
/* Purpose:  Defines Trace B mode : Store, View or Blank                     */
/*===========================================================================*/           
ViStatus _VI_FUNC AVULIS_Set_TraceBMode (ViSession instrID, ViInt16 traceBMode)
{
    ViStatus AVULIS_status = VI_SUCCESS;  
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (AVULIS_invalidViInt16Range (traceBMode,0,3))             
        return VI_ERROR_PARAMETER2;
        
    Fmt (wrtBuffer,"%s<%s\r\n",s_traceBMode[traceBMode]);   
    if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
        return AVULIS_status;
    
    return AVULIS_status;                                          
}

/*===========================================================================*/
/* Function: Set Detector Mode                                               */           
/* Purpose:  Specifies detection mode: posi-nega, posi-peak...               */
/*===========================================================================*/           
ViStatus _VI_FUNC AVULIS_Set_TraceDetector (ViSession instrID, ViInt16 detectorMode) 
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;

    if (AVULIS_invalidViInt16Range (detectorMode,0,3))             
        return VI_ERROR_PARAMETER2;

    Fmt (wrtBuffer,"%s<%s\r\n",s_detectorMode[detectorMode]);   
    if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
        return AVULIS_status;
    
    return AVULIS_status;                                          
}

/*===========================================================================*/
/* Function: Set Limit Line                                                  */
/* Purpose:  Defines or clears a whole limit line table                      */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_Set_LimitLine (ViSession instrID, ViBoolean limitLine,
                                                           ViString LL_FreqTimeTable,
                                                           ViString LL_LevelTable)
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    ViReal64 freqtime[51];
    ViReal64 level[51];
    ViInt16 i=0;
    AVULIS_ptrFeatures featuresPtr;
    
    if ((AVULIS_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVULIS_status;
    
    if (AVULIS_invalidViBooleanRange (limitLine))             
        return VI_ERROR_PARAMETER2;
    
    for (i=0; i<51; i++) {
        freqtime[i]=0;
        level[i]=0;
    }
    
    Scan(LL_FreqTimeTable,"%s>%51f[x]",freqtime);
    Scan(LL_LevelTable,"%s>%51f[x]",level);
    
    Fmt (Buffer,"%s<%s,%s\r\n",s_LLType[featuresPtr->span],s_deleteLL[limitLine]);   
    if ((AVULIS_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVULIS_status;
    i=0;
    while ( (i<=48)&&((freqtime[i]!=0)||(freqtime[i+1]!=0)||(freqtime[i+2]!=0)) ) {
        Fmt (Buffer,"%s<%s%f%s,%fDB\r\n",s_setLL[limitLine],freqtime[i],
                                         s_positionUnit[featuresPtr->span],level[i]);   
        if ((AVULIS_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
            return AVULIS_status;
        i++;   
    }   
     
    return AVULIS_status; 
}

/*===========================================================================*/
/* Function: Activate Limit Line                                             */
/* Purpose:  Show Limit Line 1 and/or Limit Line 2                           */
/*           Choose between absolute or relative along the two axis          */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_Activate_LimitLine (ViSession instrID,ViBoolean LL1Status,
                                                               ViBoolean LL2Status,
                                                               ViBoolean levelXaxis,
                                                               ViBoolean levelYaxis,
                                                               ViBoolean passfail)
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;

    if (AVULIS_invalidViBooleanRange (LL1Status))             
        return VI_ERROR_PARAMETER2;
        
    if (AVULIS_invalidViBooleanRange (LL2Status))             
        return VI_ERROR_PARAMETER3;
        
    if (AVULIS_invalidViBooleanRange (levelXaxis))             
        return VI_ERROR_PARAMETER4;
    
    if (AVULIS_invalidViBooleanRange (levelYaxis))             
        return VI_ERROR_PARAMETER5;
        
    if (AVULIS_invalidViBooleanRange (passfail))             
        return VI_ERROR_PARAMETER6;
        
        
    
    Fmt (wrtBuffer,"%s<%s,%s,%s,%s,%s\r\n",s_showLL1[LL1Status],s_showLL2[LL2Status],
                                           s_levelX[levelXaxis],s_levelY[levelYaxis],
                                           s_LLpassfail[passfail]);   
    if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
        return AVULIS_status;
    
    return AVULIS_status;                                          
}

/*===========================================================================*/
/* Function: Set Display Line                                                */           
/* Purpose:  Specifies display line position and show or hide it             */
/*===========================================================================*/           
ViStatus _VI_FUNC AVULIS_Set_DisplayLine (ViSession instrID,ViBoolean showDL,
                                                            ViReal64 DL_Level)
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    AVULIS_ptrFeatures featuresPtr;
    ViUInt32 retCount=0;

    if ((AVULIS_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVULIS_status;

    if (AVULIS_invalidViBooleanRange (showDL))             
        return VI_ERROR_PARAMETER2;
    
    if (showDL == VI_ON) { /* activation */
        if (AVULIS_invalidViReal64Range (DL_Level,featuresPtr->DL_LevelMin,
                                                  featuresPtr->DL_LevelMax))             
            return VI_ERROR_PARAMETER3;
        Fmt (wrtBuffer,"%s<%s %f%s\r\n",s_showDL[showDL],DL_Level,
                                        s_levelUnit[featuresPtr->refUnit]);   
        if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
            return AVULIS_status;
    }
    else {  /* desactivation */
        Fmt (wrtBuffer,"%s<%s\r\n",s_showDL[showDL]);   
        if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
            return AVULIS_status;
    } 
     
    return AVULIS_status;                                          
}

/*===========================================================================*/
/* Function: Set Measurement Window                                          */
/* Purpose:  Sets center position, Window width and shows or hides           */
/*           mesurement window                                               */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_Set_MeasurementWindow (ViSession instrID,ViBoolean windowStatus,
                                                                  ViReal64 Xposition,
                                                                  ViReal64 Xwidth,
                                                                  ViReal64 Yposition,
                                                                  ViReal64 Ywidth)
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViReal64 levelmax,levelmin;
    ViUInt32 retCount=0;
    AVULIS_ptrFeatures featuresPtr;
    
    if ((AVULIS_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVULIS_status;
    
    if (AVULIS_invalidViBooleanRange (windowStatus))             
        return VI_ERROR_PARAMETER2;
        
    if (windowStatus == VI_OFF) {
        Fmt (wrtBuffer,"%s<%s\r\n",s_showWindow[windowStatus]);   
        if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
            return AVULIS_status;
    }
    else {
        levelmax=Yposition + 0.5*Ywidth;
        levelmin=Yposition - 0.5*Ywidth;
        Fmt (wrtBuffer,"%s<%s%f%s,%s%f%s,%s%f%s,%s%f%s,%s\r\n",
             s_positionWindow[0],Xposition,s_positionUnit[featuresPtr->span],
             s_positionWindow[1],Xwidth,s_positionUnit[featuresPtr->span],
             s_positionWindow[2],levelmax,s_levelUnit[featuresPtr->refUnit],
             s_positionWindow[3],levelmin,s_levelUnit[featuresPtr->refUnit],
             s_showWindow[windowStatus]);   
        if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
            return AVULIS_status;
    } 
    
    return AVULIS_status;
}  

/*===========================================================================*/
/* Function: Set Normal Marker                                               */
/* Purpose:  sets the normal marker position                                 */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_Set_NormalMarker (ViSession instrID,ViReal64 position)
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    AVULIS_ptrFeatures featuresPtr;
    
    if ((AVULIS_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVULIS_status;
    
    Fmt (wrtBuffer,"%s<%s%f%s\r\n",s_mkrNormal[0],position,s_positionUnit[featuresPtr->span]);   
    if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
        return AVULIS_status;   
    
    return AVULIS_status;
}

/*===========================================================================*/
/* Function: Set Delta Marker                                                */
/* Purpose:  Sets the delta marker position                                  */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_Set_DeltaMarker (ViSession instrID, ViReal64 position)
{
    ViStatus AVULIS_status = VI_SUCCESS; 
    ViByte wrtBuffer[BUFFER_SIZE];      
    ViUInt32 retCount=0;
    ViInt16 status, fixedMarkerStatus, dividedMarkerStatus;
    ViChar message[40];
    ViReal64 MKRNpos, MKRDpos, level, dividedMarkerValue;
    AVULIS_ptrFeatures featuresPtr;
    
    if ((AVULIS_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVULIS_status;
        
    AVULIS_Read_MarkerStatus (instrID, &status, message);
    AVULIS_Read_DeltaMarkerOptions (instrID, &fixedMarkerStatus, message, &dividedMarkerStatus, &dividedMarkerValue);
    
    if ( (status == 2)&&(fixedMarkerStatus == 0) ) {  /* normal and delta markers are already active and delta not fixed */
    
        AVULIS_Read_NormalMarker (instrID, &MKRNpos, &level);
        AVULIS_Read_DeltaMarker (instrID, &MKRDpos, &level);
        
        Fmt (wrtBuffer,"%s<%s%f%s\r\n",s_mkrDelta[0],0.0,s_positionUnit[featuresPtr->span]);   
        if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
            return AVULIS_status; 
        
        Fmt (wrtBuffer,"%s<%s%f%s\r\n",s_mkrNormal[0],MKRDpos,s_positionUnit[featuresPtr->span]);   
        if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
            return AVULIS_status;
            
        Fmt (wrtBuffer,"%s<%s%f%s\r\n",s_mkrDelta[0],MKRNpos,s_positionUnit[featuresPtr->span]);   
        if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
            return AVULIS_status;
    }       
    
    Fmt (wrtBuffer,"%s<%s%f%s\r\n",s_mkrDelta[0],position,s_positionUnit[featuresPtr->span]);   
    if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
        return AVULIS_status;
    
    return AVULIS_status;
}

/*===========================================================================*/
/* Function: Set delta marker options                                        */
/* Purpose:  Sets the display mode and activates or deactivates delta        */
/*           marker fixed and 1/delta marker                                 */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_Set_DeltaMarkerOptions (ViSession instrID,ViBoolean divided,
                                                                   ViBoolean fixed)
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];      
    ViUInt32 retCount=0;

    if (AVULIS_invalidViBooleanRange (divided))             
        return VI_ERROR_PARAMETER2;
       
    if (AVULIS_invalidViBooleanRange (fixed))             
        return VI_ERROR_PARAMETER3;
       
    Fmt (wrtBuffer,"%s<%s,%s\r\n",s_1_deltaMkr[divided],s_fixedMkrDelta[fixed]);   
    if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
        return AVULIS_status;
          
    return AVULIS_status;
}

/*===========================================================================*/
/* Function : Set Marker Trace                                               */
/* Purpose : chooses the trace on which the markers will be active           */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_Set_MarkerTrace (ViSession instrID, ViBoolean markerTrace)
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (AVULIS_invalidViBooleanRange (markerTrace))             
        return VI_ERROR_PARAMETER2;
    
    Fmt (Buffer,"%s<%s\r\n",s_mkrTrace[markerTrace]);   
    if ((AVULIS_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVULIS_status;
       
    return AVULIS_status;
}    
    
/*===========================================================================*/
/* Function: Activate Multimarkers                                           */
/* Purpose:  activates the multimarkers                                      */
/*===========================================================================*/                                  
ViStatus _VI_FUNC AVULIS_Activate_Multimarkers (ViSession instrID)
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    Fmt (Buffer,"%s<%s\r\n",s_multiMkrStatus[0]);   
    if ((AVULIS_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVULIS_status;
    
    return AVULIS_status;                              
} 

/*===========================================================================*/
/* Function: Set Multimarkers                                                */
/* Purpose:  sets the Multi_Markers (frequency or time)                      */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_Set_Multimarkers (ViSession instrID, ViString multiMkrStatus,
                                                              ViString multiMkrPositions)
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    ViReal64 freqtime[6];
    ViInt16 i=0;
    AVULIS_ptrFeatures featuresPtr;
    
    if ((AVULIS_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVULIS_status;
    
    for (i=0; i<6; i++) {
        freqtime[i]=0;
    }
    
    Scan(multiMkrStatus,"%s>%6d[b2x]",featuresPtr->MKRMstatus);
    Scan(multiMkrPositions,"%s>%6f[x]",freqtime);
    
    for (i=0; i<6; i++) {
        if (featuresPtr->MKRMstatus[i]!= 0) {
            Fmt (wrtBuffer,"%s<%s%f%s\r\n",s_multiMkrON[i],freqtime[i],s_positionUnit[featuresPtr->span]);   
            if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
                return AVULIS_status;
        }
        else {
            Fmt (wrtBuffer,"%s<%s\r\n",s_multiMkrOFF[i]);   
            if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
                return AVULIS_status;
        }
    }
    
    return AVULIS_status; 
    
}

/*===========================================================================*/
/* Function: Set ActiveMultimarker                                           */
/* Purpose:  chooses the active Multi-Marker                                 */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_Set_ActiveMultiMarker (ViSession instrID, ViInt16 index)
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    ViInt16 activeMultiMarker, status = 0;
    ViChar message[100];
    ViReal64 MKRMpositions[6];
    ViReal64 MKRDposition, MKRNposition, deltaPosition;
    ViReal64 MKRMlevels[6];
    ViReal64 MKRDlevel, MKRNlevel;
    
    AVULIS_ptrFeatures featuresPtr;
    
    if ((AVULIS_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVULIS_status;
        
    if (AVULIS_invalidViInt16Range (index,1,6))             
        return VI_ERROR_PARAMETER2;
        
    AVULIS_Read_MarkerStatus (instrID, &status, message);       
    
    AVULIS_Read_MultiMarkerActive (instrID, &activeMultiMarker);
    
    AVULIS_Read_NormalMarker (instrID, &MKRNposition, &MKRNlevel);
    
    AVULIS_Read_Multimarkers (instrID, MKRMpositions,&MKRDposition,
                                       MKRMlevels,&MKRDlevel);
                                       
    deltaPosition = MKRNposition - MKRMpositions[activeMultiMarker - 1];                                   
                                       
    if (featuresPtr->MKRMstatus[index-1]!= 0) {                                   
        Fmt (Buffer,"%s<%s %f%s\r\n",s_multiMkrON[index-1],MKRMpositions[index-1] + (status - 1)*deltaPosition,s_positionUnit[featuresPtr->span]);   
        if ((AVULIS_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
            return AVULIS_status; 
    }
    
    else {
        return VI_ERROR_MARKERINACTIVE;
    }   
    
    return AVULIS_status; 
}

/*===========================================================================*/
/* Function: Delete Markers                                                  */
/* Purpose:  deletes all the markers active (normal, delta, multimarkers)    */
/*===========================================================================*/                                  
ViStatus _VI_FUNC AVULIS_Delete_Markers (ViSession instrID)
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    Fmt (Buffer,"%s<%s\r\n",s_deleteMarker[0]);   
    if ((AVULIS_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVULIS_status;
    
    return AVULIS_status;                              
} 

/*===========================================================================*/
/* Function: Set Peak Search Options                                         */
/* Purpose:  Sets the features of the peak search                            */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_Set_PeakSearchOptions (ViSession instrID,ViBoolean continuousSearch,
                                                                  ViBoolean signalTrack,
                                                                  ViInt16 peakRange,
                                                                  ViReal64 deltaY)
{
    ViStatus AVULIS_status = VI_SUCCESS; 
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    AVULIS_ptrFeatures featuresPtr;
   
    if ((AVULIS_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVULIS_status;

    if (AVULIS_invalidViBooleanRange (continuousSearch))             
        return VI_ERROR_PARAMETER2;
       
    if (AVULIS_invalidViBooleanRange (signalTrack))             
        return VI_ERROR_PARAMETER3;
    
    if (AVULIS_invalidViInt16Range (peakRange,0,2))             
        return VI_ERROR_PARAMETER4;
       
    if (AVULIS_invalidViReal64Range(deltaY,featuresPtr->deltaYmin,
                                           featuresPtr->deltaYmax))
        return VI_ERROR_PARAMETER5;
        
    Fmt (wrtBuffer,"%s<%s,%s,%s,%s%fENT\r\n",s_continuousSearch[continuousSearch],
                                             s_signalTrack[signalTrack],
                                             s_peakRange[peakRange],s_peakRange[3],deltaY);   
    if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
        return AVULIS_status;
       
    return AVULIS_status;
}

/*===========================================================================*/
/* Function: Search Peak                                                     */
/* Purpose:  set execute peak marker type                                    */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_Search_Peak (ViSession instrID, ViInt16 typeSearch)
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (AVULIS_invalidViInt16Range (typeSearch,0,5))             
        return VI_ERROR_PARAMETER2;
    
    Fmt (wrtBuffer,"%s<%s\r\n",s_peakSearch[typeSearch]);   
    if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
        return AVULIS_status;
       
    return AVULIS_status;
}

/*===========================================================================*/
/* Function: execute Peak to                                                 */
/* Purpose:  peak to setting                                                 */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_Execute_PeakTo (ViSession instrID, ViBoolean peakTo)
{
    ViStatus AVULIS_status = VI_SUCCESS; 
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (AVULIS_invalidViBooleanRange (peakTo))             
        return VI_ERROR_PARAMETER2;
        
    Fmt (wrtBuffer,"%s<%s\r\n",s_peakTo[peakTo]);   
    if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
        return AVULIS_status;
       
    return AVULIS_status;
}
 
/*===========================================================================*/
/* Function: Marker Delta to setting                                         */
/* Purpose:  marker delta to setting                                         */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_Set_MarkerDeltaTo (ViSession instrID, ViInt16 deltaTo)
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;

    if (AVULIS_invalidViInt16Range (deltaTo,0,3))             
        return VI_ERROR_PARAMETER2;

    Fmt (wrtBuffer,"%s<%s\r\n",s_markerDeltaTo[deltaTo]);   
    if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
        return AVULIS_status;
    
    return AVULIS_status;
}

/*===========================================================================*/
/* Function: Take Sweep                                                      */
/* Purpose:  Sets sweep mode & starts sweeping                               */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_Take_Sweep (ViSession instrID, ViInt16 startSweep)
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
  
    if (AVULIS_invalidViInt16Range (startSweep,0,3))             
        return VI_ERROR_PARAMETER2;
     
    Fmt (wrtBuffer,"%s<%s\r\n",s_sweepMode[startSweep]);   
    if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
        return AVULIS_status;   
       
    return AVULIS_status;
}  

/*===========================================================================*/
/* Function: Execute Trace Math                                              */
/* Purpose:  Trace Math execution                                            */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_Execute_TraceMath (ViSession instrID, ViInt16 traceMath)
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (AVULIS_invalidViInt16Range (traceMath,0,4))             
        return VI_ERROR_PARAMETER2;
    
    Fmt (wrtBuffer,"%s<%s\r\n",s_traceMath[traceMath]);   
    if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
        return AVULIS_status;   
 
    return AVULIS_status;  
}

/*===========================================================================*/
/* Function: Normalizing Mode                                                */
/* Purpose:  Normalize Mode execution                                        */
/*===========================================================================*/ 
ViStatus _VI_FUNC AVULIS_NormalizingMode (ViSession instrID, ViInt16 mode)
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (AVULIS_invalidViInt16Range (mode,0,3))             
        return VI_ERROR_PARAMETER2;
        
    Fmt (wrtBuffer,"%s<%s\r\n",s_NormMode[mode]);   
    if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
        return AVULIS_status;    
    
    return AVULIS_status;
}    

/*===========================================================================*/
/* Function: Start/Stop Trace Average                                        */
/* Purpose:  Starts stops trace average                                      */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_Control_TraceAverage (ViSession instrID, ViInt16 traceAverage)
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (AVULIS_invalidViInt16Range (traceAverage,0,3))             
        return VI_ERROR_PARAMETER2;
        
    Fmt (wrtBuffer,"%s<%s\r\n",s_traceAverage[traceAverage]);   
    if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
        return AVULIS_status;    
        
    return AVULIS_status;  
}

/*===========================================================================*/
/* Function: Read Frequency Settings                                         */
/* Purpose:  Reads a frequency that you have chosen                          */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_Read_FreqSettings (ViSession instrID, ViInt16 parameter,
                                                               ViPReal64 value)
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (AVULIS_invalidViInt16Range (parameter,0,3))             
        return VI_ERROR_PARAMETER2;
    
    Fmt (Buffer,"%s<%s\r\n",s_readFreq[parameter]);   
    if ((AVULIS_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVULIS_status;
    
    if ((AVULIS_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return AVULIS_status;
       
    if (Scan(Buffer,"%s>%f",value)!=1) { 
        AVULIS_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVULIS_status;
    }
     
    return AVULIS_status;
}

/*===========================================================================*/
/* Function: Read Frequency Offset                                           */
/* Purpose:  Reads the frequency offset status and value                     */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_Read_FreqOffset (ViSession instrID, ViPInt16 status,
                                                             ViPReal64 value)
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    Fmt (Buffer,"%s<%s\r\n",s_readFreq[4]);   
    if ((AVULIS_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVULIS_status;
    
    if ((AVULIS_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return AVULIS_status;
       
    if (Scan(Buffer,"%s>%d[b2]%f",status,value)!=2) { 
        AVULIS_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVULIS_status;
    }
     
    return AVULIS_status;
}

/*===========================================================================*/
/* Function: Read Frequency Step Size                                        */
/* Purpose:  returns the frequency step size and says whether this parameter */
/*           is controled  by the user or by the analyzer                    */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_Read_FreqStepSize (ViSession instrID, ViPReal64 value,
                                                               ViPInt16 controlType)
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    Fmt (Buffer,"%s<%s\r\n",s_readFreqStepSize[0]);   
    if ((AVULIS_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVULIS_status;
    
    if ((AVULIS_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return AVULIS_status;
       
    if (Scan(Buffer,"%s>%f",value)!=1) { 
        AVULIS_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVULIS_status;
    }
     
    Fmt (Buffer,"%s<%s\r\n",s_readFreqStepSize[1]);   
    if ((AVULIS_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVULIS_status;
    
    if ((AVULIS_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return AVULIS_status;
          
    if (Scan(Buffer,"%s>%d[b2]", controlType)!=1) { 
        AVULIS_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVULIS_status;
    }
      
    return AVULIS_status;
}

/*===========================================================================*/
/* Function: Read Frequency Counter                                          */
/* Purpose:  returns the frequency counter value and says whether the counter*/
/*           is active or inactive                                           */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_Read_FreqCounter (ViSession instrID,ViPReal64 counterValue,
                                                             ViPInt16 counterStatus)
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    Fmt (Buffer,"%s<%s\r\n",s_readFreqCounter[0]);   
    if ((AVULIS_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVULIS_status;
    
    if ((AVULIS_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return AVULIS_status;
       
    if (Scan(Buffer,"%s>%d[b2]", counterStatus)!=1) { 
        AVULIS_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVULIS_status;
    }
     
    Fmt (Buffer,"%s<%s\r\n",s_readFreqCounter[1]);   
    if ((AVULIS_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVULIS_status;
    
    if ((AVULIS_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return AVULIS_status;
          
    if (Scan(Buffer,"%s>%f", counterValue)!=1) { 
        AVULIS_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVULIS_status;
    }
      
    return AVULIS_status;
}

/*===========================================================================*/
/* Function: Read  Reference Level                                           */
/* Purpose:  returns the value (without unit) of the reference level         */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_Read_ReferenceLevel(ViSession instrID, ViPReal64 refLevelValue)
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    Fmt (Buffer,"%s<%s\r\n",s_readRefLevel[0]);   
    if ((AVULIS_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVULIS_status;
    
    if ((AVULIS_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return AVULIS_status;
       
    if (Scan(Buffer,"%s>%f", refLevelValue)!=1) { 
        AVULIS_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVULIS_status;
    }
     
    return AVULIS_status;
}

/*===========================================================================*/
/* Function: Read Reference Level Offset                                     */
/* Purpose:  returns the value (unit is always dB) of the reference level    */
/* offset and says whether this offset is active or not                      */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_Read_RefLevelOffset (ViSession instrID,ViPReal64 value,
                                                                ViPInt16 activation)
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    Fmt (Buffer,"%s<%s\r\n",s_readRefLevelOffset[0]);   
    if ((AVULIS_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVULIS_status;
    
    if ((AVULIS_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return AVULIS_status;
       
    if (Scan(Buffer,"%s>%d[b2x]%f",activation, value)!=2) { 
        AVULIS_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVULIS_status;
    }
     
    return AVULIS_status;
}

/*===========================================================================*/
/* Function: Read Reference Unit                                             */
/* Purpose:  returns the reference unit                                      */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_Read_RefUnit (ViSession instrID,ViPInt16 unitNumber,
                                                         ViChar _VI_FAR unitName[])
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    Fmt (Buffer,"%s<%s\r\n",s_readRefUnit[0]);   
    if ((AVULIS_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVULIS_status;
    
    if ((AVULIS_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return AVULIS_status;
       
    if (Scan(Buffer,"%s>%d[b2]", unitNumber)!=1) { 
        AVULIS_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVULIS_status;
    }
    
    Fmt(unitName,"%s<%s", s_readUnitName[*unitNumber]);  
     
    return AVULIS_status;
} 
                            
/*===========================================================================*/
/* Function: Read Scale                                                      */
/* Purpose:  reads the scale : status number and text                        */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_Read_Scale (ViSession instrID, ViPInt16 status,
                                                        ViChar _VI_FAR scaleName[])
{
    ViStatus AVULIS_status = VI_SUCCESS;
    AVULIS_ptrFeatures featuresPtr;
    
    if ((AVULIS_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVULIS_status;
        
    *status = featuresPtr->scale;
    
    Fmt(scaleName,"%s<%s", s_readScale[*status]);
    
    return AVULIS_status;
}

/*===========================================================================*/
/* Function: Read Attenuation                                                */
/* Purpose:  returns the input attenuation and says whether this parameter   */
/*           is controled by the user or by the analyzer                     */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_Read_Attenuation (ViSession instrID,ViPReal64 value,
                                                             ViPInt16 controlType)
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    Fmt (Buffer,"%s<%s\r\n",s_readAtt[0]);   
    if ((AVULIS_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVULIS_status;
    
    if ((AVULIS_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return AVULIS_status;
       
    if (Scan(Buffer,"%s>%f", value)!=1) { 
        AVULIS_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVULIS_status;
    }
     
    Fmt (Buffer,"%s<%s\r\n",s_readAtt[1]);   
    if ((AVULIS_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVULIS_status;
    
    if ((AVULIS_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return AVULIS_status;
          
    if (Scan(Buffer,"%s>%d[b2]", controlType)!=1) { 
        AVULIS_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVULIS_status;
    }
      
    return AVULIS_status;
}

/*===========================================================================*/
/* Function: Read RBW and VBW                                                */
/* Purpose:  reads the resolution or video bandwith and says whether this    */
/*           parameter is controlled by the analyzer or by the user          */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_Read_RBWandVBW (ViSession instrID, ViBoolean parameter,
                                                            ViPReal64 value,
                                                            ViPInt16 controlType)
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (AVULIS_invalidViBooleanRange (parameter))             
        return VI_ERROR_PARAMETER2;
    
    Fmt (Buffer,"%s<%s\r\n",s_readRBWandVBW[parameter]);   
    if ((AVULIS_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVULIS_status;
    
    if ((AVULIS_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return AVULIS_status;
       
    if (Scan(Buffer,"%s>%f", value)!=1) { 
        AVULIS_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVULIS_status;
    }
     
    Fmt (Buffer,"%s<%s\r\n",s_readRBWandVBW[parameter+2]);   
    if ((AVULIS_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVULIS_status;
    
    if ((AVULIS_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return AVULIS_status;
        
    if (Scan(Buffer,"%s>%d[b2]", controlType)!=1) { 
        AVULIS_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVULIS_status;
    } 
     
    return AVULIS_status;
}

/*===========================================================================*/
/* Function: Read  Sweep Time                                                */
/* Purpose:  reads the sweep time value and says whether we are in auto or   */
/*           manual control                                                  */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_Read_SweepTime (ViSession instrID, ViPReal64 timeValue,
                                                            ViPInt16 controlType)
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    Fmt (Buffer,"%s<%s\r\n",s_readSweepTime[0]);   
    if ((AVULIS_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVULIS_status;
    
    if ((AVULIS_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return AVULIS_status;
       
    if (Scan(Buffer,"%s>%f",timeValue)!=1) { 
        AVULIS_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVULIS_status;
    }
     
    Fmt (Buffer,"%s<%s\r\n",s_readSweepTime[1]);   
    if ((AVULIS_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVULIS_status;
    
    if ((AVULIS_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return AVULIS_status;
       
    if (Scan(Buffer,"%s>%d[b2]", controlType)!=1) { 
        AVULIS_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVULIS_status;
    } 
     
    return AVULIS_status;
}                                                       

/*===========================================================================*/
/* Function: Read Sweep Mode                                                 */
/* Purpose:  returns the sweep Mode: Normal&Full, Single&Full, Normal&Window */
/*           or Single&Window                                                */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_Read_SweepMode (ViSession instrID, ViPInt16 status,
                                                            ViChar _VI_FAR sweepMode[])
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    Fmt (Buffer,"%s<%s\r\n",s_readSweepMode[0]);   
    if ((AVULIS_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVULIS_status;
    
    if ((AVULIS_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return AVULIS_status;
       
    if (Scan(Buffer,"%s>%d[b2]", status)!=1) { 
        AVULIS_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVULIS_status;
    }
     
    if (*status==0)
        Fmt(sweepMode,"%s<%s",s_readSweepMode[1]);
    else if (*status==20)
        Fmt(sweepMode,"%s<%s",s_readSweepMode[2]);   
    else if (*status==1)
        Fmt(sweepMode,"%s<%s",s_readSweepMode[3]); 
    else if (*status==21)
        Fmt(sweepMode,"%s<%s",s_readSweepMode[4]);
     
    return AVULIS_status;
}

/*===========================================================================*/
/* Function: Read Trigger                                                    */
/* Purpose:  returns the informations about the trigger                      */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_Read_Trigger (ViSession instrID, ViPInt16 triggerStatus,
                                                          ViChar _VI_FAR triggerName[],
                                                          ViPReal64 triggerFeature)
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    ViInt16 index=0;
    
    /*searchs the trigger used*/
    
    Fmt (Buffer,"%s<%s\r\n",s_readTrigger[0]);   
    if ((AVULIS_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVULIS_status;
    
    if ((AVULIS_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return AVULIS_status;
       
    if (Scan(Buffer,"%s>%d[b2]", triggerStatus)!=1) { 
        AVULIS_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVULIS_status;
    }
    
    if (*triggerStatus==0)
        Fmt(triggerName,"%s<%s",s_readTrigger[1]);
    else if (*triggerStatus==2) {
        Fmt(triggerName,"%s<%s",s_readTrigger[3]);
        index = 3;
    }    
    else if (*triggerStatus==3)
        Fmt(triggerName,"%s<%s",s_readTrigger[2]); 
    else if (*triggerStatus==4) {
        Fmt(triggerName,"%s<%s",s_readTrigger[4]);
        index = 4;
    }    
    else if (*triggerStatus==5) {
        Fmt(triggerName,"%s<%s",s_readTrigger[5]);    
        index = 5;
    }    
       
    Fmt (Buffer,"%s<%s\r\n",s_readTrigger[index+3]);   
    if ((AVULIS_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVULIS_status;
    
    if ((AVULIS_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return AVULIS_status;
        
    if (Scan(Buffer,"%s>%f", triggerFeature)!=1) { 
        AVULIS_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVULIS_status;
    }   
     
    return AVULIS_status;
}

/*===========================================================================*/
/* Function: Read Trigger Position                                           */
/* Purpose:  returns the position of the trigger                             */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_Read_TriggerPosition (ViSession instrID, ViPInt16 triggerPosition)
                                                          
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    ViInt16 index=0;
    
    Fmt (Buffer,"%s<%s\r\n",s_readTrigger[9]);   
    if ((AVULIS_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVULIS_status;
    
    if ((AVULIS_status = viRead (instrID, Buffer, 10, &retCount)) < 0)
        return AVULIS_status;
       
    if (Scan(Buffer,"%s>%d[b2]", triggerPosition)!=1) { 
        AVULIS_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVULIS_status;
    }
    
    return AVULIS_status;
}

/*===========================================================================*/
/* Function: Read Display Line                                               */
/* Purpose:  returns the position and the status of the Display Line         */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_Read_DisplayLine(ViSession instrID, ViPReal64 level,
                                                             ViPInt16 status,
                                                             ViChar _VI_FAR message[])
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    AVULIS_ptrFeatures featuresPtr;
    ViUInt32 retCount=0;

    if ((AVULIS_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVULIS_status;
       
    Fmt (Buffer,"%s<%s\r\n",s_readDisplayLine[0]);   
    if ((AVULIS_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVULIS_status;
    
    if ((AVULIS_status = viRead (instrID, Buffer, 20, &retCount)) < 0)
        return AVULIS_status;
       
    if (Scan(Buffer,"%s>%d[b2x]%f", status, level)!=2) { 
        AVULIS_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVULIS_status;
    }
     
    Fmt(message,"%s<%s",s_textONOFF[*status]);
    
    return AVULIS_status;
}

/*===========================================================================*/
/* Function : Read Limit Line Options                                        */
/* Purpose : returns the options used with the limit lines                   */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_Read_LimitLineOptions (ViSession instrID, ViBoolean parameter,
                                                                   ViPInt16 levelStatus,
                                                                   ViChar _VI_FAR levelMessage[])
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (AVULIS_invalidViBooleanRange (parameter))             
        return VI_ERROR_PARAMETER2;
       
    Fmt (Buffer,"%s<%s\r\n",s_readLLoptions[parameter]);   
    if ((AVULIS_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVULIS_status;
    
    if ((AVULIS_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return AVULIS_status;
       
    if (Scan(Buffer,"%s>%d[b2]", levelStatus)!=1) { 
        AVULIS_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVULIS_status;
    }
     
    Fmt(levelMessage,"%s<%s",s_readLLmessage[*levelStatus]);
    
    return AVULIS_status;
}  

/*===========================================================================*/
/* Function: Read Measurement Window                                         */
/* Purpose:  returns the features and the status of the measurement window   */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_Read_MeasurementWindow (ViSession instrID,ViPInt16 status,
                                                                   ViChar _VI_FAR message[],
                                                                   ViPReal64 Xposition,
                                                                   ViPReal64 Xwidth,
                                                                   ViPReal64 Yposition,
                                                                   ViPReal64 Ywidth)
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;

    /*reads the status of the measurement window*/
    
    Fmt (Buffer,"%s<%s\r\n",s_readMeasWindow[4]);   
    if ((AVULIS_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVULIS_status;
    
    if ((AVULIS_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return AVULIS_status;
       
    if (Scan(Buffer,"%s>%d[b2]", status)!=1) { 
        AVULIS_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVULIS_status;
    }
    
    Fmt(message,"%s<%s",s_textONOFF[*status]);
    
    /*reads the position of the window along the X-axis*/
    
    Fmt (Buffer,"%s<%s\r\n",s_readMeasWindow[0]);   
    if ((AVULIS_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVULIS_status;
    
    if ((AVULIS_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return AVULIS_status;
       
    if (Scan(Buffer,"%s>%f", Xposition)!=1) { 
        AVULIS_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVULIS_status;
    }
     
    /*reads the width of the window along the X-axis*/ 
     
    Fmt (Buffer,"%s<%s\r\n",s_readMeasWindow[1]);   
    if ((AVULIS_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVULIS_status;
    
    if ((AVULIS_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return AVULIS_status;
       
    if (Scan(Buffer,"%s>%f", Xwidth)!=1) { 
        AVULIS_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVULIS_status;
    }
    
    /*reads the position of the window along the Y-axis*/
    
    Fmt (Buffer,"%s<%s\r\n",s_readMeasWindow[2]);   
    if ((AVULIS_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVULIS_status;
    
    if ((AVULIS_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return AVULIS_status;
       
    if (Scan(Buffer,"%s>%f", Yposition)!=1) { 
        AVULIS_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVULIS_status;
    }
     
    /*reads the width of the window along the Y-axis*/ 
     
    Fmt (Buffer,"%s<%s\r\n",s_readMeasWindow[3]);   
    if ((AVULIS_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVULIS_status;
    
    if ((AVULIS_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return AVULIS_status;
       
    if (Scan(Buffer,"%s>%f", Ywidth)!=1) { 
        AVULIS_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVULIS_status;
    }    
    
    return AVULIS_status;
}

/*===========================================================================*/
/* Function: Read Peak Options                                               */
/* Purpose:  reads the features of the peak search                           */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_Read_Peak_Options (ViSession instrID, ViPInt16 contPeakStatus,
                                                               ViChar _VI_FAR contPeakMessage[],
                                                               ViPInt16 signalTrackStatus,
                                                               ViChar _VI_FAR signalTrackMessage[],
                                                               ViPReal64 deltaY)
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    /*reads the status of the continuous peak search*/
    
    Fmt (Buffer,"%s<%s\r\n",s_readPeakOptions[0]);   
    if ((AVULIS_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVULIS_status;
    
    if ((AVULIS_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return AVULIS_status;
       
    if (Scan(Buffer,"%s>%d[b2]", contPeakStatus)!=1) { 
        AVULIS_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVULIS_status;
    }
    
    Fmt(contPeakMessage,"%s<%s",s_textONOFF[*contPeakStatus]);
    
    /*reads the status of the signal track*/
    
    Fmt (Buffer,"%s<%s\r\n",s_readPeakOptions[1]);   
    if ((AVULIS_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVULIS_status;
    
    if ((AVULIS_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return AVULIS_status;
       
    if (Scan(Buffer,"%s>%d[b2]", signalTrackStatus)!=1) { 
        AVULIS_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVULIS_status;
    }
    
    Fmt(signalTrackMessage,"%s<%s",s_textONOFF[*signalTrackStatus]);
    
    
    /*reads the value of deltaY*/
    
    Fmt (Buffer,"%s<%s\r\n",s_readPeakOptions[2]);   
    if ((AVULIS_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVULIS_status;
    
    if ((AVULIS_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return AVULIS_status;
        
    if (Scan(Buffer,"%s>%f", deltaY)!=1) {                                  
        AVULIS_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVULIS_status;
    }
     
    return AVULIS_status;
}

/*===========================================================================*/
/* Function: Read Trace Status                                               */
/* Purpose:  returns the trace A or trace B status                           */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_Read_TraceStatus (ViSession instrID, ViBoolean parameter,
                                                              ViPInt16 status,
                                                              ViChar _VI_FAR message[])
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViUInt32 retCount;
    ViInt16 status1=0;
    ViByte Buffer[BUFFER_SIZE];
    
    if (AVULIS_invalidViBooleanRange (parameter))             
        return VI_ERROR_PARAMETER2; 
 
    Fmt (Buffer,"%s<%s\r\n",s_readTraceStatus[parameter]);   
    if ((AVULIS_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVULIS_status;
    
    if ((AVULIS_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return AVULIS_status;
       
    if (Scan(Buffer,"%s>%d[b2]", status)!=1) { 
        AVULIS_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVULIS_status;
    }
    
    if ((*status/256)>=1)  status1 = (*status/256)+6;
    else status1 = *status;
     
    Fmt(message,"%s<%s",s_readTraceMessage[status1+parameter*3]); 
    
    return AVULIS_status; 
}

/*===========================================================================*/
/* Function: Read Detection Mode                                             */
/* Purpose:  returns the detector mode                                       */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_Read_DetectionMode (ViSession instrID, ViPInt16 status,
                                                                ViChar _VI_FAR detectorMode[])
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    Fmt (Buffer,"%s<%s\r\n",s_readDetectorMode[0]);   
    if ((AVULIS_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVULIS_status;
    
    if ((AVULIS_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return AVULIS_status;
       
    if (Scan(Buffer,"%s>%d[b2]", status)!=1) { 
        AVULIS_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVULIS_status;
    }
     
    Fmt(detectorMode,"%s<%s",s_readDetectorMode[*status+1]); 
    
    return AVULIS_status; 
}

/*===========================================================================*/
/* Function: Limit Line Judgment                                             */           
/* Purpose:  returns if the test with the limit line(s) is OK                */
/*===========================================================================*/    
ViStatus _VI_FUNC AVULIS_Read_LLjudgment (ViSession instrID, ViPInt16 result,
                                                             ViChar _VI_FAR message[]) 
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    Fmt (Buffer,"%s<%s\r\n",s_LLjudgement[0]);   
    if ((AVULIS_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVULIS_status;
    
    if ((AVULIS_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return AVULIS_status;
        
    if (Scan(Buffer,"%s>%d[b2]", result)!=1) { 
        AVULIS_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVULIS_status;
    }
     
    Fmt(message,"%s<%s",s_judgementText[*result]); 
    
    return AVULIS_status; 
}

/*===========================================================================*/
/* Function: Read Limit Line Status                                          */
/* Purpose:  returns the status of the Limit Line                            */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_Read_LimitLineStatus (ViSession instrID, ViBoolean parameter,
                        ViPInt16 status, ViChar _VI_FAR message[])
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViUInt32 retCount;
    ViByte Buffer[BUFFER_SIZE];
    
    if (AVULIS_invalidViBooleanRange (parameter))             
        return VI_ERROR_PARAMETER2; 
 
    Fmt (Buffer,"%s<%s\r\n",s_readLLstatus[parameter]);   
    if ((AVULIS_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVULIS_status;
    
    if ((AVULIS_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return AVULIS_status;
       
    if (Scan(Buffer,"%s>%d[b2]", status)!=1) { 
        AVULIS_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVULIS_status;
    }
     
    Fmt(message,"%s<%s",s_textONOFF[*status]); 
    
    return AVULIS_status; 
}    

/*===========================================================================*/
/* Function: Read Marker Status                                              */
/* Purpose:  reads the marker status number -> text                          */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_Read_MarkerStatus (ViSession instrID, ViPInt16 status,
                                                               ViChar _VI_FAR message[])
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    Fmt (Buffer,"%s<%s\r\n",s_mkrNDsearch[0]);   
    if ((AVULIS_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVULIS_status;
       
    if ((AVULIS_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return AVULIS_status;
       
    if (Scan(Buffer,"%s>%d[b2]",status)!=1) { 
        AVULIS_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVULIS_status;
    }   
    
    Fmt(message,"%s<%s", s_mkrNDstatus[*status]);  
    
    return AVULIS_status;
}

/*===========================================================================*/
/* Function: Read Delta Marker Options                                       */
/* Purpose:  returns the options status of the delta marker                  */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_Read_DeltaMarkerOptions (ViSession instrID,
                                                  ViPInt16 fixedMarkerStatus,
                                                  ViChar _VI_FAR fixedMarkerMessage[],
                                                  ViPInt16 dividedMarkerStatus,
                                                  ViPReal64 dividedMarkerValue)
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViUInt32 retCount=0;
    ViByte Buffer[BUFFER_SIZE];
    
    /*about the fixed delta marker*/
    
    Fmt (Buffer,"%s<%s\r\n",s_readMKDoptions[0]);   
    if ((AVULIS_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVULIS_status;
    
    if ((AVULIS_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return AVULIS_status;
       
    if (Scan(Buffer,"%s>%d[b2]", fixedMarkerStatus)!=1) { 
        AVULIS_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVULIS_status;
    }
     
    Fmt(fixedMarkerMessage,"%s<%s",s_textONOFF[*fixedMarkerStatus]);
    
    /*about the 1/delta marker*/
    
    Fmt (Buffer,"%s<%s\r\n",s_readMKDoptions[1]);   
    if ((AVULIS_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVULIS_status;
    
    if ((AVULIS_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return AVULIS_status;
       
    if (Scan(Buffer,"%s>%d[b2x]%f",dividedMarkerStatus,dividedMarkerValue)!=2) { 
        AVULIS_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVULIS_status;
    }  
    
    return AVULIS_status; 
}    
    

/*===========================================================================*/
/* Function: Read Marker Trace Status                                        */
/* Purpose:  returns the trace on which the markers are placed               */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_Read_MarkerTraceStatus (ViSession instrID,ViPInt16 status,
                                                                   ViChar _VI_FAR message[])
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViUInt32 retCount;
    ViByte Buffer[BUFFER_SIZE];
    
    Fmt (Buffer,"%s<%s\r\n",s_readMarkerTraceStatus[0]);   
    if ((AVULIS_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVULIS_status;
    
    if ((AVULIS_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return AVULIS_status;
       
    if (Scan(Buffer,"%s>%d[b2]", status)!=1) { 
        AVULIS_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVULIS_status;
    }
     
    Fmt(message,"%s<%s",s_readMarkerTraceStatus[*status+1]); 
    
    return AVULIS_status;
}
 
/*===========================================================================*/
/* Function: Read Multi Marker Status                                        */
/* Purpose:  reads the multi marker status & gives text message              */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_Read_MultimarkersStatus (ViSession instrID,ViPInt16 status,
                                                                    ViChar _VI_FAR message[])
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    Fmt (Buffer,"%s<%s\r\n",s_readmultiMkrStatus[0]);   
    if ((AVULIS_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVULIS_status;
       
    if ((AVULIS_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return AVULIS_status;
       
    if (Scan(Buffer,"%s>%d[b2]",status)!=1) { 
        AVULIS_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVULIS_status;
    }   
    
    Fmt(message,"%s<%s", s_multiMkrMessage[*status]);  
    
    return AVULIS_status;
}

/*===========================================================================*/
/* Function: Read Trace Data                                                 */
/* Purpose:  returns trace A or B in an array                                */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_Read_TraceData (ViSession instrID, ViBoolean trace,
                                                            ViBoolean format,
                                                            ViInt16 _VI_FAR traceArray[])
{
    ViStatus AVULIS_status = VI_SUCCESS;
    AVULIS_ptrFeatures featuresPtr;
    ViInt16 i;
    ViUInt32 retCount;
    ViByte dataBuffer[1405];
    ViByte wrtBuffer[BUFFER_SIZE];
    
    if (AVULIS_invalidViBooleanRange (trace))             
        return VI_ERROR_PARAMETER2;
        
    if (AVULIS_invalidViBooleanRange (format))             
        return VI_ERROR_PARAMETER3;
        
    if ((AVULIS_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVULIS_status;
       
    for (i=0;i<1405;i++) {
        dataBuffer[i]=0;
    } 
       
    if (format == VI_ON) { /*Binary Transfert*/
        Fmt (wrtBuffer,"%s<%s\r\n",s_traceBinary[trace]);     
        if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return AVULIS_status;
          
        if ((AVULIS_status = viRead (instrID,dataBuffer,(2*701)+2,&retCount)) < 0)
            return AVULIS_status;
       
        for (i=0;i<701;i++) {
            traceArray[i]=256*dataBuffer[2*i]+dataBuffer[2*i+1] ;
        }
    }
    
    else { /*ASCII Transfert*/ 
        Fmt (wrtBuffer,"%s<%s\r\n",s_traceASCII[trace]);     
        if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return AVULIS_status;
        for (i=0;i<701;i++) {
            if ((AVULIS_status = viRead (instrID, dataBuffer, 7, &retCount)) < 0)
                return AVULIS_status;
            if (Scan(dataBuffer,"%s>%d[b2]", &traceArray[i])!=1) { 
                AVULIS_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
                return AVULIS_status;
            }   
        }
    }   
     
  return AVULIS_status; 
}

/*===========================================================================*/
/* Function: Set Trace Accuracy                                              */
/* Purpose:  chooses the value of the level's accuracy                       */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_Set_TraceAccuracy (ViSession instrID, ViBoolean accuracy)
  
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (AVULIS_invalidViBooleanRange (accuracy))             
        return VI_ERROR_PARAMETER2;
    
    Fmt (Buffer,"%s<%s\r\n",s_traceAccuracy[accuracy]);   
    if ((AVULIS_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVULIS_status;
        
    return AVULIS_status;
}

/*===========================================================================*/
/* Function: Read Trace Accuracy                                             */
/* Purpose:  returns the value of the level's accuracy                       */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_Read_TraceAccuracy (ViSession instrID, ViPInt16 accuracy)
  
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    ViInt16 status;
    
    Fmt (Buffer,"%s<%s\r\n",s_traceAccuracy[2]);   
    if ((AVULIS_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVULIS_status;
        
    if ((AVULIS_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return AVULIS_status;
       
    if (Scan(Buffer,"%s>%d[b2]", &status)!=1) { 
        AVULIS_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVULIS_status;
    }
    
    if (status==0) *accuracy=341;
    else *accuracy=2721;
        
    return AVULIS_status;
}

/*===========================================================================*/
/* Function: read normal marker                                              */
/* Purpose:  reads the normal marker position and level (without unit)       */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_Read_NormalMarker (ViSession instrID,ViPReal64 MKRNposition,
                                            ViPReal64 MKRNlevel)
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    /*reads the position of the normal marker*/
    
    Fmt (Buffer,"%s<%s\r\n",s_mkrNDsearch[1]);   
    if ((AVULIS_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVULIS_status;
    
    if ((AVULIS_status = viRead (instrID, Buffer, 20, &retCount)) < 0)
        return AVULIS_status;
       
    if (Scan(Buffer,"%s>%f", MKRNposition)!=1) { 
        AVULIS_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVULIS_status;
    }
     
    /*reads the level of the normal marker*/ 
     
    Fmt (Buffer,"%s<%s\r\n",s_mkrNDsearch[2]);   
    if ((AVULIS_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVULIS_status;
    
    if ((AVULIS_status = viRead (instrID, Buffer, 20, &retCount)) < 0)
        return AVULIS_status;
       
    if (Scan(Buffer,"%s>%f", MKRNlevel)!=1) { 
        AVULIS_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVULIS_status;
    }

    return AVULIS_status;
}

/*===========================================================================*/
/* Function: read delta marker                                               */
/* Purpose:  reads the position and level (without unit) of the delta marker */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_Read_DeltaMarker (ViSession instrID,ViPReal64 MKRDposition,
                                           ViPReal64 MKRDlevel)
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    ViInt16 offset;
    AVULIS_ptrFeatures featuresPtr;
    
    if ((AVULIS_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVULIS_status;
    
    /* returns the delta marker position */
    
    Fmt (Buffer,"%s<%s\r\n",s_multiMkrSearch[0]);   
    if ((AVULIS_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVULIS_status;

    if ((AVULIS_status = viRead (instrID, Buffer, 130, &retCount)) < 0)
        return AVULIS_status;
       
    offset = retCount-11-8*featuresPtr->span ;
    
    if (Scan(Buffer,"%s[i*]>%f",offset,MKRDposition)!=1) { 
        AVULIS_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVULIS_status;
    }
     
    /* returns the delta marker level */
    
    Fmt (Buffer,"%s<%s\r\n",s_multiMkrSearch[1]);   
    if ((AVULIS_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVULIS_status;
       
    if ((AVULIS_status = viRead (instrID, Buffer, 130, &retCount)) < 0)
        return AVULIS_status;
    
    offset = retCount - 14;
    
    if (Scan(Buffer,"%s[i*]>%f",offset,MKRDlevel)!=1) { 
        AVULIS_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVULIS_status;
    }   

    return AVULIS_status;
}
 
/*===========================================================================*/
/* Function: Read Multi Markers                                              */
/* Purpose:  reads the positions and levels of the 6 multimarkers            */
/*           and the delta marker                                            */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_Read_Multimarkers (ViSession instrID,ViReal64 _VI_FAR MKRMpositions[6],
                                                              ViPReal64 MKRDposition,
                                                              ViReal64 _VI_FAR MKRMlevels[6],
                                                              ViPReal64 MKRDlevel)
                                          
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    /* returns the multimarkers positions */
    
    Fmt (Buffer,"%s<%s\r\n",s_multiMkrSearch[0]);   
    if ((AVULIS_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVULIS_status;

    if ((AVULIS_status = viRead (instrID, Buffer, 130, &retCount)) < 0)
        return AVULIS_status;
    
    if (Scan(Buffer,"%s>%6f[x]%f",MKRMpositions,MKRDposition)!=2) { 
        AVULIS_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVULIS_status;
    }
     
    /* returns the multimarkers levels*/
    
    Fmt (Buffer,"%s<%s\r\n",s_multiMkrSearch[1]);   
    if ((AVULIS_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVULIS_status;
       
    if ((AVULIS_status = viRead (instrID, Buffer, 130, &retCount)) < 0)
        return AVULIS_status;
       
    if (Scan(Buffer,"%s>%6f[x]%f",MKRMlevels,MKRDlevel)!=2) { 
        AVULIS_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVULIS_status;
    }   

    return AVULIS_status;
}

/*===========================================================================*/
/* Function: Read MultiMarkerActive                                          */
/* Purpose:  searches the active Multi-Marker                                */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_Read_MultiMarkerActive (ViSession instrID, ViPInt16 index)
                                      
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    ViInt32 bytesRead;
    ViInt16 i = 0, markerStatus, fixedMarkerStatus, dividedMarkerStatus;
    ViReal64 MKRNposition, delta, deltaMin = 10e9;
    ViReal64 MKRMpositions[6];
    ViReal64 MKRDposition;
    ViReal64 MKRMlevels[6];
    ViReal64 MKRDlevel, MKRstepsize;
    ViChar message[40];
    ViReal64 MKRNpos, MKRDpos, level, dividedMarkerValue;
    
    AVULIS_Read_MarkerStatus (instrID, &markerStatus, message);
    AVULIS_Read_DeltaMarkerOptions (instrID, &fixedMarkerStatus, message, &dividedMarkerStatus, &dividedMarkerValue);
    AVULIS_Read_Multimarkers (instrID, MKRMpositions, &MKRDposition, MKRMlevels, &MKRDlevel);  
    
    if ( (markerStatus == 2)&&(fixedMarkerStatus == 1) ) {  /* fixed delta markers */
    
    	// MKR step size = 0
    	AVULIS_Set_FreqStepSize (instrID, 0, 0);
    	
    	// MKR -> MKR step size
    	AVULIS_WriteInstrData (instrID, s_markerStepSize[0]);
    	
    	// returns the new MKR step size
    	AVULIS_WriteInstrData (instrID, s_markerStepSize[1]);
    	AVULIS_readInstrData (instrID, 50, Buffer, &bytesRead);
    	Scan(Buffer,"%s>%f",&MKRstepsize);
                                                           
    	// Compare MKR step size and MKRMpositions[i] to determine the active marker 
    	for (i = 0; i < 6; i++) {
    		delta = fabs ((MKRDposition + MKRMpositions[i]) - MKRstepsize) ;
    		if ( delta < deltaMin ) {
    			deltaMin = delta;
    			*index = i + 1;
    		}
    	}
    	
    	// MKR step size = AUTO
    	AVULIS_Set_FreqStepSize (instrID, 1, 0);
    
    }    
    
    else {
    
    	Fmt (Buffer,"%s<%s\r\n",s_mkrNDsearch[1]);   
    	if ((AVULIS_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        	return AVULIS_status;
    
    	if ((AVULIS_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        	return AVULIS_status;
       
	    if (Scan(Buffer,"%s>%f", &MKRNposition)!=1) { 
	        AVULIS_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
	        return AVULIS_status;
	    }
     
	    while ( (i<6)&&((MKRMpositions[i]!=MKRNposition)||(MKRMlevels[i]==0)) ) {
	        i++;
	    } 
    
   		*index = i+1;
   		
   	}	
    
    return AVULIS_status; 
}

/*===========================================================================*/
/* Function: Write To Instrument                                             */
/* Purpose:  This function writes a command string to the instrument.        */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_WriteInstrData (ViSession instrID, ViString writeBuffer)
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    Fmt (wrtBuffer,"%s<%s\r\n",writeBuffer);     
    if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return AVULIS_status; 
    
    return AVULIS_status;
}

/*===========================================================================*/
/* Function: Read Instr Data                                                 */
/* Purpose:  This function reads the output buffer of the instrument.        */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_readInstrData (ViSession instrID, ViInt32 numBytes,
                                                           ViChar _VI_FAR readBuffer[],
                                                           ViPInt32 bytesRead)
{
    ViStatus AVULIS_status = VI_SUCCESS;
    *bytesRead = 0L;
        
    if ((AVULIS_status = viRead (instrID, readBuffer, numBytes, bytesRead)) < 0)
        return AVULIS_status;

    return AVULIS_status;
}

/*===========================================================================*/
/* Function: Reset                                                           */
/* Purpose:  This function resets the instrument.                            */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_reset (ViSession instrSession)
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    AVULIS_ptrFeatures featuresPtr;
    
    if ((AVULIS_status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, &featuresPtr)) < 0)
        return AVULIS_status;
    
    /*  Initialize the instrument to a known state.  */
    Fmt (wrtBuffer,"%s<%s\r\n",s_reset[0]);     
    if ((AVULIS_status = viWrite(instrSession,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return AVULIS_status;
   
    if ((AVULIS_status = AVULIS_defaultInstrSetup (instrSession, featuresPtr->instrumentType)) < 0)  
        return AVULIS_status;
        
    return AVULIS_status;
}

/*===========================================================================*/
/* Function: Self-Test                                                       */
/* Purpose:  This function executes the instrument self-test and returns     */
/*           the result. The self test function is not supported by this     */
/*           instrument, and this function returns the warning               */
/*           VI_WARN_NSUP_SELF_TEST. (Self Test not supported)               */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_selfTest (ViSession instrID, ViPInt16 selfTestResult,
                                                      ViChar _VI_FAR selfTestMessage[])
{
    ViStatus AVULIS_status = VI_WARN_NSUP_SELF_TEST;
    
    Fmt (selfTestMessage, "%s<Self-Test Not Supported");
    *selfTestResult = -1;
    
    return AVULIS_status;
}

/*===========================================================================*/
/* Function: Error Query                                                     */
/* Purpose:  This function queries the instrument error queue, and returns   */
/*           the result.                                                     */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_errorQuery (ViSession instrID, ViPInt32 errorCode,
                                                        ViChar _VI_FAR errorMessage[])
{
    ViUInt32 retCnt = 0;
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    Fmt (Buffer,"%s<%s\r\n",s_errorQuery[0]);     
    if ((AVULIS_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount)) < 0)
        return AVULIS_status;
       
    if ((AVULIS_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return AVULIS_status;
       
    if (Scan(Buffer,"%s>%d[b4]", errorCode)!=1) { 
        AVULIS_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVULIS_status;
    }
     
    Fmt (errorMessage,"%s<%d[b4]", *errorCode); 

    return AVULIS_status;
}

/*===========================================================================*/
/* Function: Error Message                                                   */
/* Purpose:  This function translates the error return value from the        */
/*           instrument driver into a user-readable string.                  */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_errorMessage (ViSession instrID, ViStatus errorCode,
                                                          ViChar _VI_FAR errorMessage[])
 
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViInt16 i;
    
    static AVULIS_tStringValPair statusDescArray[] = 
    {
        {VI_WARN_NSUP_SELF_TEST,    "WARNING: Self-test not supported"},
        {VI_ERROR_PARAMETER1,        "ERROR: Parameter 1 out of range"},
        {VI_ERROR_PARAMETER2,        "ERROR: Parameter 2 out of range"},
        {VI_ERROR_PARAMETER3,        "ERROR: Parameter 3 out of range"},
        {VI_ERROR_PARAMETER4,        "ERROR: Parameter 4 out of range"},
        {VI_ERROR_PARAMETER5,        "ERROR: Parameter 5 out of range"},
        {VI_ERROR_PARAMETER6,        "ERROR: Parameter 6 out of range"},
        {VI_ERROR_PARAMETER7,        "ERROR: Parameter 7 out of range"},
        {VI_ERROR_PARAMETER8,        "ERROR: Parameter 8 out of range"},
        {VI_ERROR_INSTR_PARAMETER9 , "ERROR: Parameter 9 out of range"},
        {VI_ERROR_INSTR_PARAMETER10, "ERROR: Parameter 10 out of range"},
        {VI_ERROR_INSTR_PARAMETER11, "ERROR: Parameter 11 out of range"},
        {VI_ERROR_INSTR_PARAMETER12, "ERROR: Parameter 12 out of range"},
        {VI_ERROR_INSTR_PARAMETER13, "ERROR: Parameter 13 out of range"},
        {VI_ERROR_INSTR_PARAMETER14, "ERROR: Parameter 14 out of range"},
        {VI_ERROR_INSTR_PARAMETER15, "ERROR: Parameter 15 out of range"},
        {VI_ERROR_FAIL_ID_QUERY,    "ERROR: Identification query failed"},
        {VI_ERROR_INV_RESPONSE,     "ERROR: Interpreting instrument response"},
        {VI_ERROR_INSTR_FILE_OPEN,  "ERROR: Opening the specified file"},
        {VI_ERROR_INSTR_FILE_WRITE, "ERROR: Writing to the specified file"},
        {VI_ERROR_INSTR_INTERPRETING_RESPONSE, "ERROR: Interpreting the instrument's response"},
        {VI_ERROR_MARKERINACTIVE,   "ERROR: the chosen multi-marker is inactive"},
        {VI_NULL, VI_NULL}
     };

    AVULIS_status = viStatusDesc (instrID, errorCode, errorMessage);
    
    if (AVULIS_status == VI_WARN_UNKNOWN_STATUS) {
        for (i=0; statusDescArray[i].stringName; i++) {
            if (statusDescArray[i].stringVal == errorCode) {
                Fmt (errorMessage, "%s<%s", statusDescArray[i].stringName);
                return (VI_SUCCESS);
            }
        }
        Fmt (errorMessage, "%s<Unknown Error 0x%x[uw8p0]", errorCode); 
        return (VI_WARN_UNKNOWN_STATUS);
    }
    
    AVULIS_status = VI_SUCCESS;
    return AVULIS_status;
}

/*===========================================================================*/
/* Function: Revision Query                                                  */
/* Purpose:  This function returns the driver and instrument revisions.      */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_revisionQuery (ViSession instrID, ViChar _VI_FAR driverRev[],
                                                           ViChar _VI_FAR instrRev[])
{
    ViUInt32 retCount = 0;
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    
    Fmt (Buffer,"%s<%s\r\n",s_instrRevision[0]);     
    if ((AVULIS_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount)) < 0)
        return AVULIS_status;
    
    if ((AVULIS_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return AVULIS_status;
       
    if (Scan(Buffer,"%s>%s", instrRev)!=1) { 
        AVULIS_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVULIS_status;
    }
     
    Fmt (driverRev, "%s<%s", AVULIS_REVISION);
    
    return AVULIS_status;
}

/*===========================================================================*/
/* Function: Calibration                                                     */
/* Purpose:  Set calibration to element                                      */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_Calibration (ViSession instrID, ViInt16 element,
                                                         ViReal64 level,
                                                         ViBoolean calCorrection,
                                                         ViBoolean freqCorrection)
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    AVULIS_ptrFeatures featuresPtr;
    ViUInt32 retCount=0;
    ViUInt32 timeout;
    
    
    if ((AVULIS_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVULIS_status;
    
    if (AVULIS_invalidViInt16Range (element,0,7))             
        return VI_ERROR_PARAMETER2;
        
    if (AVULIS_invalidViReal64Range (level,featuresPtr->calLevelMin,
                                           featuresPtr->calLevelMax))          
        return VI_ERROR_PARAMETER3;

    if (AVULIS_invalidViBooleanRange (calCorrection))             
        return VI_ERROR_PARAMETER4;
    
    if (AVULIS_invalidViBooleanRange (freqCorrection))             
        return VI_ERROR_PARAMETER5;
        
    /* stores the value of the timeout in variable timeout and then disables the timeout */
    if ((AVULIS_status = viGetAttribute (instrID,VI_ATTR_TMO_VALUE,&timeout)) < 0)
        return AVULIS_status;
    
    if ((AVULIS_status = viSetAttribute (instrID, VI_ATTR_TMO_VALUE, 1800000)) < 0)
            return AVULIS_status;
       
    /* element to calibrate*/  
    Fmt (wrtBuffer,"%s<%s%fDB,%s\r\n",s_calLevel[0],level,s_calibration[element]);     
    if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return AVULIS_status;     

    /* calibration and frequency correction */ 
    Fmt (wrtBuffer,"%s<%s,%s\r\n",s_freqCorrection[freqCorrection],
                                  s_calCorrection[calCorrection]);     
    if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return AVULIS_status;
    
    /* disables the calibration's signal */
    Fmt (wrtBuffer,"%s<%s\r\n",s_calLevel[1]);     
    if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return AVULIS_status;  
    
    /*  enables the timeout and sets its value to its last value */  
    if ((AVULIS_status = viSetAttribute (instrID, VI_ATTR_TMO_VALUE, timeout)) < 0)
        return AVULIS_status;
        
    return AVULIS_status;
}

/*===========================================================================*/
/* Function: Display label                                                   */
/* Purpose:  selects label                                                   */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_Display_Label (ViSession instrID, ViBoolean showLabel,
                                                           ViString label)
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (AVULIS_invalidViBooleanRange (showLabel))             
        return VI_ERROR_PARAMETER2;
       
    if (showLabel == VI_ON) {
        Fmt (wrtBuffer,"%s<%s,%s%s/\r\n",s_label[showLabel+1],s_label[showLabel+2],label);     
        if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return AVULIS_status; 
    }
    else {
        Fmt (wrtBuffer,"%s<%s,%s\r\n",s_label[showLabel],s_label[showLabel+1]);     
        if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return AVULIS_status;
    }
    
    return AVULIS_status;   
}

/*===========================================================================*/
/* Function : Read Status Byte                                               */
/* purpose : controls the status byte register                               */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_Read_StatusByte (ViSession instrID, ViPInt16 statusByte)
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViUInt16 status = 0;
    ViByte Buffer[BUFFER_SIZE];
    
    AVULIS_status = viReadSTB (instrID, &status);
    * statusByte = (ViInt16) status;
    
    return AVULIS_status;
}

/*===========================================================================*/
/* Function : Clear Status Byte                                              */
/* purpose : clears the status byte register                                 */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_Clear_StatusByte (ViSession instrID)
{
 
    ViStatus AVULIS_status = VI_SUCCESS;
    ViUInt32 retCount = 0;
    ViByte Buffer[BUFFER_SIZE];
    
    Fmt (Buffer,"%s<%s\r\n",s_clearStatusByte[0]);     
    if ((AVULIS_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount)) < 0)
        return AVULIS_status;
       
    return AVULIS_status;
    
}

/*===========================================================================*/
/* Function : Convert Frequency                                              */
/* Purpose : converts the frequency value returned in Hz by the analyzer     */
/*           in the value with the best unit (Hz, kHz, MHz or GHz)           */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_Convert_Frequency (ViSession instrID,ViReal64 inputValue,
                                                              ViPReal64 outputValue,
                                                              ViChar _VI_FAR frequencyUnit[])
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViReal64 denom[]={1,1000,1000000,1000000000},interim[3];
    ViInt16 i;
    
    for (i=0;i<3;i++) {
        interim[i] = inputValue / denom[i+1];
    }
    
    i=0; 
    while ( (i<3)&&(fabs(interim[i])>=1) ) {
        i++;
    }
     
    *outputValue = inputValue / denom[i];
    
    Fmt(frequencyUnit,"%s<%s",s_freqUnit[i]); 
    
    return AVULIS_status;
} 
 
/*===========================================================================*/
/* Function : Convert Time                                                   */
/* Purpose : converts the time value returned in second(s) by the analyzer   */
/*           in the value with the best unit (ns, us, ms or s)               */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_Convert_Time (ViSession instrID, ViReal64 inputValue,
                                                          ViPReal64 outputValue,
                                                          ViChar _VI_FAR timeUnit[])
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViReal64 compare[]={1000000000,1000000,1000,1};
    ViInt16 i;
    
    i=0;
    while ((i<=2)&&(fabs(inputValue)*compare[i]>=1000)) {
        i++;
    }
     
    *outputValue = inputValue*compare[i];
    Fmt(timeUnit,"%s<%s",s_timeUnit[i]);
    
    return AVULIS_status;
 
}  

/*===========================================================================*/
/* Function: Initialize card                                                 */
/* Purpose:  Initialize card on drive A or B                                 */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_Init_MemoryCard (ViSession instrID, ViBoolean drive)
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (AVULIS_invalidViBooleanRange (drive))          
        return VI_ERROR_PARAMETER2;
       
    Fmt (wrtBuffer,"%s<%s\r\n",s_initCard[drive]);     
    if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return AVULIS_status;      
    
    return AVULIS_status;
}

/*===========================================================================*/
/* Function: select Memory Card                                              */
/* Purpose:  selects memory card drive                                       */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_Select_MemoryCard (ViSession instrID, ViBoolean drive)
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (AVULIS_invalidViBooleanRange (drive))
        return VI_ERROR_PARAMETER2;
       
    Fmt (wrtBuffer,"%s<%s\r\n",s_selectCard[drive]);     
    if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return AVULIS_status;    
    
    return AVULIS_status;
}

/*===========================================================================*/
/* Function: Copy Memory Card                                                */
/* Purpose:  Copy card A: to card B:                                         */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_Copy_MemoryCard (ViSession instrID, ViBoolean parameter)
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (AVULIS_invalidViBooleanRange (parameter))
        return VI_ERROR_PARAMETER2;
    
    Fmt (wrtBuffer,"%s<%s\r\n",s_copyCard[parameter]);     
    if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return AVULIS_status;
            
    return AVULIS_status;
}

/*===========================================================================*/
/* Function: recall a file                                                   */
/* Purpose:  allows to restore information saved on a memory card            */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_Recall_File (ViSession instrID, ViBoolean drive,
                                                         ViString fileName)
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (AVULIS_invalidViBooleanRange (drive))             
        return VI_ERROR_PARAMETER2; 
    
    Fmt (wrtBuffer,"%s<%s%s/\r\n",s_recallFile[drive],fileName);     
    if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return AVULIS_status; 
    
    return AVULIS_status;
}

/*===========================================================================*/
/* Function: save a file                                                     */
/* Purpose:  allows to save information on a memory card                     */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_Save_File (ViSession instrID,ViBoolean drive,ViBoolean format,
                                    ViString fileName,ViBoolean setup,ViInt16 trace,
                                    ViInt16 limit,ViBoolean normalize,ViBoolean antenna,
                                    ViBoolean IDlist)
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (AVULIS_invalidViBooleanRange (drive))             
        return VI_ERROR_PARAMETER2;
    if (AVULIS_invalidViBooleanRange (format))             
        return VI_ERROR_PARAMETER3; 
    if (AVULIS_invalidViBooleanRange (setup))             
        return VI_ERROR_PARAMETER5; 
    if (AVULIS_invalidViInt16Range (trace,0,3))             
        return VI_ERROR_PARAMETER6; 
    if (AVULIS_invalidViInt16Range (limit,0,3))             
        return VI_ERROR_PARAMETER7; 
    if (AVULIS_invalidViBooleanRange (normalize))             
        return VI_ERROR_PARAMETER8;
    if (AVULIS_invalidViBooleanRange (antenna))             
        return VI_ERROR_INSTR_PARAMETER9; 
    if (AVULIS_invalidViBooleanRange (IDlist))             
        return VI_ERROR_INSTR_PARAMETER10;
        
    // setup, ID list     
    Fmt (wrtBuffer,"%s<%s,%s\r\n",s_saveSetup[2*format+setup],s_saveIDlist[2*format+IDlist]);     
    if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return AVULIS_status;
        
    // trace
    Fmt (wrtBuffer,"%s<%s\r\n",s_saveTrace[4*format+trace]);     
    if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return AVULIS_status; 
    
    // limit line
    Fmt (wrtBuffer,"%s<%s\r\n",s_saveLimit[4*format+limit]);     
    if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return AVULIS_status; 
        
    if (format == VI_ON) {  // BINARY format - normalize and antenna correction can be also saved    
        Fmt (wrtBuffer,"%s<%s,%s\r\n",s_saveNormalize[normalize],s_saveAntenna[antenna]);     
        if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return AVULIS_status;
    }       
    
    // save the file on the memory card chosen with the format selected 
    Fmt (wrtBuffer,"%s<%s%s/\r\n",s_saveFile[2*format+drive],fileName);     
    if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return AVULIS_status; 
    
    return AVULIS_status;
}

/*======================================================================*/
/* Function: Set TR Antenna Correction                                  */
/* Purpose:  Activates or deactivates the antenna correction function   */
/*           for the antenna TR1722 and TR1711                          */
/*======================================================================*/ 
ViStatus _VI_FUNC AVULIS_Set_TR_AntennaCorrection (ViSession instrID, ViInt16 antennaChoice)
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (AVULIS_invalidViInt16Range (antennaChoice,0,2))             
        return VI_ERROR_PARAMETER2;
        
    Fmt (wrtBuffer,"%s<%s\r\n",s_antennaAC[antennaChoice]);   
        if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
            return AVULIS_status;    
        
    return AVULIS_status;    
}        

/*======================================================================*/
/* Function: Set User Antenna Correction                                */
/* Purpose:  Activates or deactivates the antenna correction function   */
/*           Can be used whatever the antenna is                        */
/*======================================================================*/
ViStatus _VI_FUNC AVULIS_Set_User_AntennaCorrection (ViSession instrID, ViBoolean status,
                                                     ViBoolean mdCorrection,
                                                     ViString  FreqTable,
                                                     ViString  LevelTable)
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    ViReal64 freq[27];
    ViReal64 level[27];
    ViInt16 i=0;
    
    if (AVULIS_invalidViBooleanRange (status))             
        return VI_ERROR_PARAMETER2;
        
    if (AVULIS_invalidViBooleanRange (mdCorrection))             
        return VI_ERROR_PARAMETER3;    
        
    if (status == VI_ON) {  /* Antenna Correction ON */
      
        for (i=0; i<27; i++) {
            freq[i]=0;
            level[i]=0;
        }
    
        Scan(FreqTable,"%s>%27f[x]",freq);
        Scan(LevelTable,"%s>%27f[x]",level);
    
        Fmt (wrtBuffer,"%s<%s\r\n",s_workAC[0]);   
        if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
            return AVULIS_status;
         
        i=0;
        while ( (i<=24)&&((freq[i]!=0)||(freq[i+1]!=0)||(freq[i+2]!=0)) ) {
            Fmt (wrtBuffer,"%s<%s%fHZ,%fDB\r\n",s_workAC[1],freq[i],level[i]);   
            if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
                return AVULIS_status;
            i++;   
        }
        
        Fmt (wrtBuffer,"%s<%s,%s\r\n",s_modeAC[mdCorrection],s_showAC[status]);   
        if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
            return AVULIS_status;
        
    }   
    
    else {    /* Antenna Correction OFF */
    
        Fmt (wrtBuffer,"%s<%s\r\n",s_showAC[status]);   
        if ((AVULIS_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
            return AVULIS_status; 
    
    }       
     
    return AVULIS_status; 
}

/*===========================================================================*/
/* Function: Read Antenna Correction Status                                  */
/* Purpose:  returns the level correction  status                            */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_Read_AntennaCorrectionStatus (ViSession instrID,ViPInt16 status,
                                                       ViChar _VI_FAR message[])
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViUInt32 retCount;
    ViByte Buffer[BUFFER_SIZE];
    ViInt16 prestatus;
    
    Fmt (Buffer,"%s<%s\r\n",s_readAC[0]);   
    if ((AVULIS_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVULIS_status;
    
    if ((AVULIS_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return AVULIS_status;
       
    if (Scan(Buffer,"%s>%d[b2]", &prestatus)!=1) { 
        AVULIS_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVULIS_status;
    }
    
    if (prestatus == 0) { 
        Fmt (Buffer,"%s<%s\r\n",s_readAC[1]);   
        if ((AVULIS_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
            return AVULIS_status;
    
        if ((AVULIS_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
            return AVULIS_status;
       
        if (Scan(Buffer,"%s>%d[b2]", &prestatus)!=1) { 
            AVULIS_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
            return AVULIS_status;
        }
        if (prestatus == 1) prestatus = 3;
    }
    
    *status = prestatus;
    Fmt(message,"%s<%s",s_readAC[*status+2]); 
    
    return AVULIS_status;
} 

/*===========================================================================*/
/* Function: Close                                                           */
/* Purpose:  This function closes the instrument.                            */
/*===========================================================================*/
ViStatus _VI_FUNC AVULIS_close (ViSession instrSession)
{
    AVULIS_ptrFeatures featuresPtr;
    ViSession rmSession;
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    Fmt (wrtBuffer,"%s<%s\r\n",s_close[0]); // Set Instr to Local Mode    
    if ((AVULIS_status = viWrite(instrSession,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return AVULIS_status;

    if ((AVULIS_status = viGetAttribute (instrSession,VI_ATTR_RM_SESSION,&rmSession))<0)
        return AVULIS_status;
    if ((AVULIS_status = viGetAttribute (instrSession,VI_ATTR_USER_DATA,&featuresPtr))<0)
        return AVULIS_status;
    
    if (featuresPtr != NULL) 
        free (featuresPtr);
    
    AVULIS_status = viClose (instrSession);
    viClose (rmSession);

    return AVULIS_status;
}

/*****************************************************************************/
/*= UTILITY ROUTINES (Non-Exportable Functions) =============================*/
/*****************************************************************************/


/*===========================================================================*/
/* Function: Boolean Value Out Of Range - ViBoolean                          */
/* Purpose:  This function checks a Boolean to see if it is equal to VI_TRUE */
/*           or VI_FALSE. If the value is out of range, the return value is  */
/*           VI_TRUE, otherwise the return value is VI_FALSE.                */
/*===========================================================================*/
ViBoolean AVULIS_invalidViBooleanRange (ViBoolean val)
{
    return ((val != VI_OFF && val != VI_ON) ? VI_ON : VI_OFF);
}

/*===========================================================================*/
/* Function: Short Signed Integer Value Out Of Range - ViInt16               */
/* Purpose:  This function checks a short signed integer value to see if it  */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean AVULIS_invalidViInt16Range (ViInt16 val, ViInt16 min, ViInt16 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Long Signed Integer Value Out Of Range - ViInt32                */
/* Purpose:  This function checks a long signed integer value to see if it   */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range,  the return value is VI_TRUE, otherwise the return    */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean AVULIS_invalidViInt32Range  (ViInt32 val, ViInt32 min, ViInt32 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Short Unsigned Integer Value Out Of Range - ViUInt16            */
/* Purpose:  This function checks a short unsigned integer value to see if it*/  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range,  the return value is VI_TRUE, otherwise the return    */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean AVULIS_invalidViUInt16Range  (ViUInt16 val, ViUInt16 min, ViUInt16 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Long Unsigned Integer Value Out Of Range - ViUInt32             */
/* Purpose:  This function checks a long unsigned integer value to see if it */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range,  the return value is VI_TRUE, otherwise the return    */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean AVULIS_invalidViUInt32Range  (ViUInt32 val, ViUInt32 min, ViUInt32 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}
/*===========================================================================*/
/* Function: Real (Float) Value Out Of Range - ViReal32                      */
/* Purpose:  This function checks a real (float) value to see if it lies     */  
/*           between a minimum and maximum value.  If the value is out of    */
/*           range, the return value is VI_TRUE, otherwise the return value  */
/*           is VI_FALSE.                                                    */
/*===========================================================================*/
ViBoolean AVULIS_invalidViReal32Range  (ViReal32 val, ViReal32 min, ViReal32 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Real (Double) Value Out Of Range - ViReal64                     */
/* Purpose:  This function checks a real (double) value to see if it lies    */  
/*           between a minimum and maximum value.  If the value is out of    */
/*           range, the return value is VI_TRUE, otherwise the return value  */
/*           is VI_FALSE.                                                    */
/*===========================================================================*/
ViBoolean AVULIS_invalidViReal64Range  (ViReal64 val, ViReal64 min, ViReal64 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Initialize Clean Up                                             */
/* Purpose:  This function is used only by the AVULIS_init function.  When   */
/*           an error is detected this function is called to close the       */
/*           open resource manager and instrument object sessions and to     */
/*           set the instrSession that is returned from AVULIS_init to       */
/*           VI_NULL.                                                        */
/*===========================================================================*/
ViStatus AVULIS_initCleanUp (ViSession openRMSession,
                             ViPSession openInstrSession, ViStatus currentStatus)
{
    viClose (*openInstrSession);
    viClose (openRMSession);
    *openInstrSession = VI_NULL;
    
    return currentStatus;
    
}

/*===========================================================================*/
/* function  Search Scale                                                    */
/* Purpose : search the scale used by the analyzer when the user begin       */
/*           the remote control                                              */
/*===========================================================================*/
ViStatus AVULIS_Search_Scale (ViSession instrSession, ViPInt16 scale)
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViUInt32 retCount;
    ViByte Buffer[BUFFER_SIZE];
    ViInt16   i, unitNumberStart, unitNumber;
    ViChar unitName[5];
    
    /* returns the unit */
    AVULIS_Read_RefUnit (instrSession, &unitNumberStart, unitName);
    
    /* returns the last logarithmic scale used */
    Fmt (Buffer,"%s<%s\r\n",s_yScale[5]);   
    if ((AVULIS_status = viWrite(instrSession,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVULIS_status;
    if ((AVULIS_status = viRead (instrSession, Buffer, 5, &retCount)) < 0)
        return AVULIS_status;
    if (Scan(Buffer,"%s>%d[b2]", scale)!=1) { 
        AVULIS_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVULIS_status;
     }
    
    /* send a new unit --> dBm */
    AVULIS_Set_RefUnit (instrSession, 0);
    
    /* we send the order to pass in linear scale */
    /* if we are in log scale --> lin scale and unit = V*/
    /* if we are in lin scale --> lin scale and unit = dBm*/
    Fmt (Buffer,"%s<%s\r\n",s_yScale[4]);   
    if ((AVULIS_status = viWrite(instrSession,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVULIS_status;
    AVULIS_Read_RefUnit (instrSession, &unitNumber, unitName);
    if (unitNumber != 0) {  /* unit = V --> log scale */ 
        Fmt (Buffer,"%s<%s\r\n",s_yScale[*scale]);   
        if ((AVULIS_status = viWrite(instrSession,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVULIS_status;
    }
    else {  /* unit = dBm --> linear scale */
        *scale = 4;
    }
    
    /* send the unit used at the beginning of this function */
    AVULIS_Set_RefUnit (instrSession, unitNumberStart);
        
    return AVULIS_status;
} 

/*****************************************************************************/
/*----------- INSERT INSTRUMENT-DEPENDENT UTILITY ROUTINES HERE -------------*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: Internal Reset                                                  */
/* Purpose:  This function resets the instrument.                            */
/*===========================================================================*/
ViStatus AVULIS_initReset (ViSession instrSession,ViUInt32 instrType)
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    /*  Initialize the instrument to a known state.  */
    Fmt (wrtBuffer,"%s<%s\r\n",s_reset[0]);     
    if ((AVULIS_status = viWrite(instrSession,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return AVULIS_status;
   
    if ((AVULIS_status = AVULIS_defaultInstrSetup (instrSession,instrType)) < 0)  
        return AVULIS_status;
        
    return AVULIS_status;
}

/*===========================================================================*/
/* Function: Default Instrument Setup                                        */
/* Purpose:  This function sends a default setup to the instrument.  This    */
/*           function is called by the AVULIS_reset operation and by the     */
/*           AVULIS_init function if the reset option has not been           */
/*           selected.  This function is useful for configuring any          */
/*           instrument settings that are required by the rest of the        */
/*           instrument driver functions such as turning headers ON or OFF   */
/*           or using the long or short form for commands, queries, and data.*/                                    
/*===========================================================================*/
ViStatus AVULIS_defaultInstrSetup (ViSession instrSession, ViUInt32 instrType)
{
    ViStatus AVULIS_status = VI_SUCCESS;
    ViUInt32 retCount = 0;
    ViByte wrtBuffer[BUFFER_SIZE];
    AVULIS_ptrFeatures featuresPtr;
    ViInt16 i;
    ViReal64 MKRMpositions[6],positionDelta,MKRMlevels[6];
    ViReal64 levelDelta, span;
    ViChar unitName[10];
    
    /* Determine if the structure has been initialized for the current VISA  */
    /* session and malloc if it has not.                                     */
    if (AVULIS_status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, &featuresPtr))
        return AVULIS_status;
    
    if (featuresPtr == NULL) 
        featuresPtr = malloc (sizeof (struct AVULIS_instrFeatures));
    
    /*delete the soft menus*/
    Fmt (wrtBuffer,"%s<%s\r\n",s_reset[1]);     
    if ((AVULIS_status = viWrite(instrSession,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return AVULIS_status;
        
    switch (instrType) {
    
        case 3641 :
            featuresPtr->frequencyLow[0] = 0;
            featuresPtr->frequencyHigh[0] = 3e9;
            featuresPtr->frequencyLow[1]= -200e6;
            featuresPtr->frequencyHigh[1] = 3e9;
            featuresPtr->frequencyLow[2] = 0;
            featuresPtr->frequencyHigh[2] = 3.2e9;
            featuresPtr->spanLow = 1e3;
            featuresPtr->spanHigh = 3.2e9;
            featuresPtr->sweepTimeMini = 50e-6;
            featuresPtr->sweepTimeMaxi = 1000;
            break;
            
        case 3661 :
            featuresPtr->frequencyLow[0] = 0;
            featuresPtr->frequencyHigh[0] = 26.5e9;
            featuresPtr->frequencyLow[1]= -200e6;
            featuresPtr->frequencyHigh[1] = 26.5e9;
            featuresPtr->frequencyLow[2] = 0;
            featuresPtr->frequencyHigh[2] = 26.7e9;
            featuresPtr->spanLow = 1e3;
            featuresPtr->spanHigh = 26.7e9;
            featuresPtr->sweepTimeMini = 50e-6;
            featuresPtr->sweepTimeMaxi = 1000;
            break;
            
        case 4941 :
            featuresPtr->frequencyLow[0] = 0;
            featuresPtr->frequencyHigh[0] = 2.2e9;
            featuresPtr->frequencyLow[1]= -200e6;
            featuresPtr->frequencyHigh[1] = 2.2e9;
            featuresPtr->frequencyLow[2] = 0;
            featuresPtr->frequencyHigh[2] = 2.4e9;
            featuresPtr->spanLow = 1e3;
            featuresPtr->spanHigh = 3.2e9;
            featuresPtr->sweepTimeMini = 50e-3;
            featuresPtr->sweepTimeMaxi = 1000;
            break;  
        
    }
    
    featuresPtr->instrumentType = instrType;
    featuresPtr->freqOffsetMin = -10e9;   /*Hz*/
    featuresPtr->freqOffsetMax = +10e9;
    featuresPtr->refLevelOffsetMin = -100; /*dB*/
    featuresPtr->refLevelOffsetMax = +100;
    featuresPtr->inputAttMin = 0;   /*dB*/
    featuresPtr->inputAttMax = 50;
    featuresPtr->gatePositionMini = 1e-6;
    featuresPtr->gatePositionMaxi = 65e-3;
    featuresPtr->gateWidthMini = 2e-6;
    featuresPtr->gateWidthMaxi = 65e-3;
    featuresPtr->averageCountMini = 2;
    featuresPtr->averageCountMaxi = 1000;
    featuresPtr->DL_LevelMin = -1000;
    featuresPtr->DL_LevelMax = 1000;
    featuresPtr->refLevelMin[0]=-89;
    featuresPtr->refLevelMin[1]=-42;
    featuresPtr->refLevelMin[2]=18;
    featuresPtr->refLevelMin[3]=24;
    featuresPtr->refLevelMin[4]=1;
    featuresPtr->refLevelMin[5]=0;
    featuresPtr->refLevelMin[6]=9.717e-6;
    featuresPtr->refLevelMin[7]=1.259e-12;
    featuresPtr->refLevelMax[0]=40;
    featuresPtr->refLevelMax[1]=87;
    featuresPtr->refLevelMax[2]=147;
    featuresPtr->refLevelMax[3]=153;
    featuresPtr->refLevelMax[4]=130;
    featuresPtr->refLevelMax[5]=0;
    featuresPtr->refLevelMax[6]=22.36;
    featuresPtr->refLevelMax[7]=10;
    featuresPtr->displayMode=0; /*displayMode->absolute*/
    featuresPtr->deltaYmin=0.1;
    featuresPtr->deltaYmax=10;
    featuresPtr->calLevelMin=-50;
    featuresPtr->calLevelMax=-20;
    
    AVULIS_Read_RefUnit (instrSession, &featuresPtr->refUnit,unitName);
    
    AVULIS_Read_FreqSettings (instrSession, 1, &span);
    if (span==0) featuresPtr->span=0;
    else featuresPtr->span=1;
    
    AVULIS_Read_Multimarkers (instrSession,MKRMpositions,&positionDelta,
                                           MKRMlevels,&levelDelta);
                                         
    for (i=0;i<6;i++) {
        if ((MKRMpositions[i]!=0)||(MKRMlevels[i]!=0)) featuresPtr->MKRMstatus[i]=1;
        else featuresPtr->MKRMstatus[i]=0;
    }
    
    
    Fmt (featuresPtr -> instrDriverRevision, "%s<%s", AVULIS_REVISION);    
    
    if (viSetAttribute (instrSession, VI_ATTR_USER_DATA, (ViUInt32)featuresPtr)!= 0)
        return AVULIS_status;
        
    /* returns the scale used by the spectrum analyzer */
    AVULIS_Search_Scale (instrSession, &featuresPtr->scale);    
       
    return AVULIS_status;
    
}

/*****************************************************************************/
/*=== END INSTRUMENT DRIVER SOURCE CODE =====================================*/
/*****************************************************************************/
